@@ -43,19 +43,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <errno.h>
 
-/* These are stored in the bfd's tdata */
-struct core_data {
-  struct user *upage;             /* core file header */
-  asection *data_section;
-  asection *stack_section;
-  asection *reg_section;
-};
+  struct trad_core_struct 
+    {
+      asection *data_section;
+      asection *stack_section;
+      asection *reg_section;
 
-#define core_hdr(bfd) (((struct core_data *) (bfd->tdata))->hdr)
-#define core_upage(bfd) (((struct core_data *) ((bfd)->tdata))->upage)
-#define core_datasec(bfd) (((struct core_data *) ((bfd)->tdata))->data_section)
-#define core_stacksec(bfd) (((struct core_data*)((bfd)->tdata))->stack_section)
-#define core_regsec(bfd) (((struct core_data *) ((bfd)->tdata))->reg_section)
+      struct user		u;
+    } *rawptr;
+
+
+#define core_upage(bfd) (&((bfd)->tdata.trad_core_data->u))
+#define core_datasec(bfd) ((bfd)->tdata.trad_core_data->data_section)
+#define core_stacksec(bfd) ((bfd)->tdata.trad_core_data->stack_section)
+#define core_regsec(bfd) ((bfd)->tdata.trad_core_data->reg_section)
 
 /* Handle 4.2-style (and perhaps also sysV-style) core dump file.  */
 
@@ -63,16 +64,13 @@ struct core_data {
 bfd_target *
 trad_unix_core_file_p (abfd)
      bfd *abfd;
+
 {
   int val;
   struct user u;
-  unsigned int reg_offset, fp_reg_offset;
   /* This struct is just for allocating two things with one zalloc, so
      they will be freed together, without violating alignment constraints. */
-  struct core_user {
-	struct core_data	coredata;
-	struct user		u;
-  } *rawptr;
+
 
   val = bfd_read ((void *)&u, 1, sizeof u, abfd);
   if (val != sizeof u)
@@ -89,34 +87,34 @@ trad_unix_core_file_p (abfd)
 
   /* Allocate both the upage and the struct core_data at once, so
      a single free() will free them both.  */
-  rawptr = (struct core_user *)bfd_zalloc (abfd, sizeof (struct core_user));
+  rawptr = (struct trad_core_struct *)bfd_zalloc (abfd, sizeof (struct trad_core_struct));
   if (rawptr == NULL) {
     bfd_error = no_memory;
     return 0;
   }
   
-  set_tdata (abfd, &rawptr->coredata);
-  core_upage (abfd) = &rawptr->u;
-  *core_upage (abfd) = u;		/* Save that upage! */
+  abfd->tdata.trad_core_data = rawptr;
+
+  rawptr->u = u; /*Copy the uarea into the tdata part of the bfd */
 
   /* Create the sections.  This is raunchy, but bfd_close wants to free
      them separately.  */
-  core_stacksec (abfd) = (asection *) zalloc (sizeof (asection));
+
+  core_stacksec(abfd) = (asection *) zalloc (sizeof (asection));
   if (core_stacksec (abfd) == NULL) {
-loser:
+  loser:
     bfd_error = no_memory;
     free ((void *)rawptr);
     return 0;
   }
   core_datasec (abfd) = (asection *) zalloc (sizeof (asection));
   if (core_datasec (abfd) == NULL) {
-loser1:
+  loser1:
     free ((void *)core_stacksec (abfd));
     goto loser;
   }
   core_regsec (abfd) = (asection *) zalloc (sizeof (asection));
   if (core_regsec (abfd) == NULL) {
-loser2:
     free ((void *)core_datasec (abfd));
     goto loser1;
   }
@@ -129,9 +127,9 @@ loser2:
   core_datasec (abfd)->flags = SEC_ALLOC + SEC_LOAD + SEC_HAS_CONTENTS;
   core_regsec (abfd)->flags = SEC_ALLOC + SEC_HAS_CONTENTS;
 
-  core_datasec (abfd)->size =  NBPG * u.u_dsize;
-  core_stacksec (abfd)->size = NBPG * u.u_ssize;
-  core_regsec (abfd)->size = NBPG * UPAGES;  /* Larger than sizeof struct u */
+  core_datasec (abfd)->_raw_size =  NBPG * u.u_dsize;
+  core_stacksec (abfd)->_raw_size = NBPG * u.u_ssize;
+  core_regsec (abfd)->_raw_size = NBPG * UPAGES; /* Larger than sizeof struct u */
 
   /* What a hack... we'd like to steal it from the exec file,
      since the upage does not seem to provide it.  FIXME.  */
@@ -148,7 +146,7 @@ loser2:
      from *u_ar0.  The other is that u_ar0 is sometimes an absolute address
      in kernel memory, and on other systems it is an offset from the beginning
      of the `struct user'.
-
+     
      As a practical matter, we don't know where the registers actually are,
      so we have to pass the whole area to GDB.  We encode the value of u_ar0
      by setting the .regs section up so that its virtual memory address
@@ -159,7 +157,7 @@ loser2:
 
   core_datasec (abfd)->filepos = NBPG * UPAGES;
   core_stacksec (abfd)->filepos = (NBPG * UPAGES) + NBPG * u.u_dsize;
-  core_regsec (abfd)->filepos = 0;	/* Register segment is the upage */
+  core_regsec (abfd)->filepos = 0; /* Register segment is the upage */
 
   /* Align to word at least */
   core_stacksec (abfd)->alignment_power = 2;
@@ -179,8 +177,9 @@ trad_unix_core_file_failing_command (abfd)
      bfd *abfd;
 {
 #ifndef NO_CORE_COMMAND
-  if (*core_upage (abfd)->u_comm)
-    return core_upage (abfd)->u_comm;
+  char *com = abfd->tdata.trad_core_data->u.u_comm;
+  if (*com)
+    return com;
   else
 #endif
     return 0;
@@ -208,8 +207,8 @@ trad_unix_core_file_matches_executable_p  (core_bfd, exec_bfd)
 #define	trad_unix_slurp_armap			bfd_false
 #define	trad_unix_slurp_extended_name_table	bfd_true
 #define	trad_unix_write_armap			(PROTO (boolean, (*),	\
-     (bfd *arch, unsigned int elength, struct orl *map, int orl_count,	\
-      int stridx))) bfd_false
+     (bfd *arch, unsigned int elength, struct orl *map, \
+      unsigned int orl_count, int stridx))) bfd_false
 #define	trad_unix_truncate_arname		bfd_dont_truncate_arname
 #define	aout_32_openr_next_archived_file	bfd_generic_openr_next_archived_file
 
@@ -247,7 +246,8 @@ trad_unix_core_file_matches_executable_p  (core_bfd, exec_bfd)
 #define trad_unix_bfd_debug_info_end		bfd_void
 #define trad_unix_bfd_debug_info_accumulate	(PROTO (void, (*),	\
 	(bfd *, struct sec *))) bfd_void
-
+#define trad_unix_bfd_get_relocated_section_contents bfd_generic_get_relocated_section_contents
+#define trad_unix_bfd_relax_section bfd_generic_relax_section
 /* If somebody calls any byte-swapping routines, shoot them.  */
 void
 swap_abort()
