@@ -1,5 +1,5 @@
 /* Everything about breakpoints, for GDB.
-   Copyright 1986, 1987, 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright 1986, 1987, 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -17,12 +17,12 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-#include <stdio.h>
-#include <ctype.h>
 #include "defs.h"
+#include <ctype.h>
 #include "symtab.h"
 #include "frame.h"
 #include "breakpoint.h"
+#include "gdbtypes.h"
 #include "expression.h"
 #include "gdbcore.h"
 #include "gdbcmd.h"
@@ -34,87 +34,122 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "language.h"
 #include <string.h>
 
+/* local function prototypes */
+
+static void
+catch_command_1 PARAMS ((char *, int, int));
+
+static void
+enable_delete_command PARAMS ((char *, int));
+
+static void
+enable_delete_breakpoint PARAMS ((struct breakpoint *));
+
+static void
+enable_once_command PARAMS ((char *, int));
+
+static void
+enable_once_breakpoint PARAMS ((struct breakpoint *));
+
+static void
+disable_command PARAMS ((char *, int));
+
+static void
+disable_breakpoint PARAMS ((struct breakpoint *));
+
+static void
+enable_command PARAMS ((char *, int));
+
+static void
+enable_breakpoint PARAMS ((struct breakpoint *));
+
+static void
+map_breakpoint_numbers PARAMS ((char *,	void (*)(struct breakpoint *)));
+
+static void
+ignore_command PARAMS ((char *, int));
+
+static int
+breakpoint_re_set_one PARAMS ((char *));
+
+static void
+delete_command PARAMS ((char *, int));
+
+static void
+clear_command PARAMS ((char *, int));
+
+static void
+catch_command PARAMS ((char *, int));
+
+static struct symtabs_and_lines
+get_catch_sals PARAMS ((int));
+
+static void
+watch_command PARAMS ((char *, int));
+
+static void
+tbreak_command PARAMS ((char *, int));
+
+static void
+break_command_1 PARAMS ((char *, int, int));
+
+static void
+mention PARAMS ((struct breakpoint *));
+
+static struct breakpoint *
+set_raw_breakpoint PARAMS ((struct symtab_and_line));
+
+static void
+check_duplicates PARAMS ((CORE_ADDR));
+
+static void
+describe_other_breakpoints PARAMS ((CORE_ADDR));
+
+static void
+breakpoints_info PARAMS ((char *, int));
+
+static void
+breakpoint_1 PARAMS ((int, int));
+
+static bpstat
+bpstat_alloc PARAMS ((struct breakpoint *, bpstat));
+
+static int
+breakpoint_cond_eval PARAMS ((char *));
+
+static void
+cleanup_executing_breakpoints PARAMS ((int));
+
+static void
+commands_command PARAMS ((char *, int));
+
+static void
+condition_command PARAMS ((char *, int));
+
+static int
+get_number PARAMS ((char **));
+
+static void
+set_breakpoint_count PARAMS ((int));
+
+
 extern int addressprint;		/* Print machine addresses? */
 extern int demangle;			/* Print de-mangled symbol names? */
 
-extern int catch_errors ();
-extern void set_next_address ();	/* ...for x/ command */
-
 /* Are we executing breakpoint commands?  */
 static int executing_breakpoint_commands;
 
-/* States of enablement of breakpoint.
-   `temporary' means disable when hit.
-   `delete' means delete when hit.  */
-
-enum enable { disabled, enabled, temporary, delete};
-
-/* Not that the ->silent field is not currently used by any commands
-   (though the code is in there if it was to be, and set_raw_breakpoint
-   does set it to 0).  I implemented it because I thought it would be
-   useful for a hack I had to put in; I'm going to leave it in because
-   I can see how there might be times when it would indeed be useful */
-
-/* This is for a breakpoint or a watchpoint.  */
-
-struct breakpoint
-{
-  struct breakpoint *next;
-  /* Number assigned to distinguish breakpoints.  */
-  int number;
-  /* Address to break at, or NULL if not a breakpoint.  */
-  CORE_ADDR address;
-  /* Line number of this address.  Redundant.  Only matters if address
-     is non-NULL.  */
-  int line_number;
-  /* Symtab of file of this address.  Redundant.  Only matters if address
-     is non-NULL.  */
-  struct symtab *symtab;
-  /* Zero means disabled; remember the info but don't break here.  */
-  enum enable enable;
-  /* Non-zero means a silent breakpoint (don't print frame info
-     if we stop here). */
-  unsigned char silent;
-  /* Number of stops at this breakpoint that should
-     be continued automatically before really stopping.  */
-  int ignore_count;
-  /* "Real" contents of byte where breakpoint has been inserted.
-     Valid only when breakpoints are in the program.  Under the complete
-     control of the target insert_breakpoint and remove_breakpoint routines.
-     No other code should assume anything about the value(s) here.  */
-  char shadow_contents[BREAKPOINT_MAX];
-  /* Nonzero if this breakpoint is now inserted.  Only matters if address
-     is non-NULL.  */
-  char inserted;
-  /* Nonzero if this is not the first breakpoint in the list
-     for the given address.  Only matters if address is non-NULL.  */
-  char duplicate;
-  /* Chain of command lines to execute when this breakpoint is hit.  */
-  struct command_line *commands;
-  /* Stack depth (address of frame).  If nonzero, break only if fp
-     equals this.  */
-  FRAME_ADDR frame;
-  /* Conditional.  Break only if this expression's value is nonzero.  */
-  struct expression *cond;
-
-  /* String we used to set the breakpoint (malloc'd).  Only matters if
-     address is non-NULL.  */
-  char *addr_string;
-  /* String form of the breakpoint condition (malloc'd), or NULL if there
-     is no condition.  */
-  char *cond_string;
-
-  /* The expression we are watching, or NULL if not a watchpoint.  */
-  struct expression *exp;
-  /* The largest block within which it is valid, or NULL if it is
-     valid anywhere (e.g. consists just of global symbols).  */
-  struct block *exp_valid_block;
-  /* Value of the watchpoint the last time we checked it.  */
-  value val;
-};
+/* Walk the following statement or block through all breakpoints.
+   ALL_BREAKPOINTS_SAFE does so even if the statment deletes the current
+   breakpoint.  */
 
 #define ALL_BREAKPOINTS(b)  for (b = breakpoint_chain; b; b = b->next)
 
+#define ALL_BREAKPOINTS_SAFE(b,tmp)	\
+	for (b = breakpoint_chain;	\
+	     b? (tmp=b->next, 1): 0;	\
+	     b = tmp)
+
 /* Chain of all breakpoints defined.  */
 
 struct breakpoint *breakpoint_chain;
@@ -145,9 +180,6 @@ CORE_ADDR default_breakpoint_address;
 struct symtab *default_breakpoint_symtab;
 int default_breakpoint_line;
 
-static void delete_breakpoint ();
-void breakpoint_auto_delete ();
-
 /* Flag indicating extra verbosity for xgdb.  */
 extern int xgdb_verbose;
 
@@ -189,6 +221,8 @@ get_number (pp)
     }
   else
     {
+      if (*p == '-')
+	++p;
       while (*p >= '0' && *p <= '9')
 	++p;
       if (p == *pp)
@@ -226,11 +260,11 @@ condition_command (arg, from_tty)
       {
 	if (b->cond)
 	  {
-	    free (b->cond);
+	    free ((PTR)b->cond);
 	    b->cond = 0;
 	  }
 	if (b->cond_string != NULL)
-	  free (b->cond_string);
+	  free ((PTR)b->cond_string);
 
 	if (*p == 0)
 	  {
@@ -298,7 +332,13 @@ End with a line saying just \"end\".\n", bnum);
 extern int memory_breakpoint_size; /* from mem-break.c */
 
 /* Like target_read_memory() but if breakpoints are inserted, return
-   the shadow contents instead of the breakpoints themselves.  */
+   the shadow contents instead of the breakpoints themselves.
+
+   Read "memory data" from whatever target or inferior we have. 
+   Returns zero if successful, errno value if not.  EIO is used
+   for address out of bounds.  If breakpoints are inserted, returns
+   shadow contents, not the breakpoints themselves.  From breakpoint.c.  */
+
 int
 read_memory_nobpt (memaddr, myaddr, len)
      CORE_ADDR memaddr;
@@ -314,7 +354,7 @@ read_memory_nobpt (memaddr, myaddr, len)
   
   ALL_BREAKPOINTS (b)
     {
-      if (b->address == 0 || !b->inserted)
+      if (b->type == bp_watchpoint || !b->inserted)
 	continue;
       else if (b->address + memory_breakpoint_size <= memaddr)
 	/* The breakpoint is entirely before the chunk of memory
@@ -391,7 +431,7 @@ insert_breakpoints ()
   int disabled_breaks = 0;
 
   ALL_BREAKPOINTS (b)
-    if (b->address != 0
+    if (b->type != bp_watchpoint
 	&& b->enable != disabled
 	&& ! b->inserted
 	&& ! b->duplicate)
@@ -444,7 +484,7 @@ remove_breakpoints ()
 #endif /* BREAKPOINT_DEBUG */
 
   ALL_BREAKPOINTS (b)
-    if (b->address != 0 && b->inserted)
+    if (b->type != bp_watchpoint && b->inserted)
       {
 	val = target_remove_breakpoint(b->address, b->shadow_contents);
 	if (val)
@@ -494,6 +534,10 @@ breakpoint_here_p (pc)
 
 /* bpstat stuff.  External routines' interfaces are documented
    in breakpoint.h.  */
+
+/* Clear a bpstat so that it says we are not at any breakpoint.
+   Also free any storage that is part of a bpstat.  */
+
 void
 bpstat_clear (bsp)
      bpstat *bsp;
@@ -509,12 +553,15 @@ bpstat_clear (bsp)
       q = p->next;
       if (p->old_val != NULL)
 	value_free (p->old_val);
-      free (p);
+      free ((PTR)p);
       p = q;
     }
   *bsp = NULL;
 }
 
+/* Return a copy of a bpstat.  Like "bs1 = bs2" but all storage that
+   is part of the bpstat is copied as well.  */
+
 bpstat
 bpstat_copy (bs)
      bpstat bs;
@@ -541,6 +588,27 @@ bpstat_copy (bs)
   return retval;
 }
 
+/* Find the bpstat associated with this breakpoint */
+
+bpstat
+bpstat_find_breakpoint(bsp, breakpoint)
+     bpstat bsp;
+     struct breakpoint *breakpoint;
+{
+  if (bsp == NULL) return NULL;
+
+  for (;bsp != NULL; bsp = bsp->next) {
+    if (bsp->breakpoint_at == breakpoint) return bsp;
+  }
+  return NULL;
+}
+
+/* Return the breakpoint number of the first breakpoint we are stopped
+   at.  *BSP upon return is a bpstat which points to the remaining
+   breakpoints stopped at (but which is not guaranteed to be good for
+   anything but further calls to bpstat_num).
+   Return 0 if passed a bpstat which does not indicate any breakpoints.  */
+
 int
 bpstat_num (bsp)
      bpstat *bsp;
@@ -560,6 +628,8 @@ bpstat_num (bsp)
     }
 }
 
+/* Modify BS so that the actions will not be performed.  */
+
 void
 bpstat_clear_actions (bs)
      bpstat bs;
@@ -588,6 +658,7 @@ cleanup_executing_breakpoints (ignore)
    location.  Any of these commands could cause the process to proceed
    beyond this point, etc.  We look out for such changes by checking
    the global "breakpoint_proceeded" after each command.  */
+
 void
 bpstat_do_actions (bsp)
      bpstat *bsp;
@@ -617,19 +688,25 @@ top:
 	    goto top;
 	}
     }
-  clear_momentary_breakpoints ();
 
   executing_breakpoint_commands = 0;
   discard_cleanups (old_chain);
 }
 
+/* Print a message indicating what happened.  Returns nonzero to
+   say that only the source line should be printed after this (zero
+   return means print the frame as well as the source line).  */
+
 int
 bpstat_print (bs)
      bpstat bs;
 {
   /* bs->breakpoint_at can be NULL if it was a momentary breakpoint
      which has since been deleted.  */
-  if (bs == NULL || bs->breakpoint_at == NULL)
+  if (bs == NULL
+      || bs->breakpoint_at == NULL
+      || (bs->breakpoint_at->type != bp_breakpoint
+	  && bs->breakpoint_at->type != bp_watchpoint))
     return 0;
   
   /* If bpstat_stop_status says don't print, OK, we won't.  An example
@@ -642,7 +719,7 @@ bpstat_print (bs)
   if (!bs->print)
     return 0;
 
-  if (bs->breakpoint_at->address != 0)
+  if (bs->breakpoint_at->type == bp_breakpoint)
     {
       /* I think the user probably only wants to see one breakpoint
 	 number, not all of them.  */
@@ -702,7 +779,7 @@ bpstat_alloc (b, cbs)
   bs->breakpoint_at = b;
   /* If the condition is false, etc., don't do the commands.  */
   bs->commands = NULL;
-  bs->momentary = b->number == -3;
+  bs->momentary = b->disposition == delete;
   bs->old_val = NULL;
   return bs;
 }
@@ -740,7 +817,7 @@ bpstat_stop_status (pc, frame_address)
   int real_breakpoint = 0;
 #endif
   /* Root of the chain of bpstat's */
-  struct bpstat__struct root_bs[1];
+  struct bpstat root_bs[1];
   /* Pointer to the last thing in the chain currently.  */
   bpstat bs = root_bs;
 
@@ -754,15 +831,18 @@ bpstat_stop_status (pc, frame_address)
 
       if (b->enable == disabled)
 	continue;
-      if (b->address != 0 && b->address != bp_addr)
+
+      if (b->type != bp_watchpoint && b->address != bp_addr)
 	continue;
 
+      /* Come here if it's a watchpoint, or if the break address matches */
+
       bs = bpstat_alloc (b, bs);	/* Alloc a bpstat to explain stop */
 
       this_bp_stop = 1;
       this_bp_print = 1;
 
-      if (b->exp != NULL)		/* Watchpoint */
+      if (b->type == bp_watchpoint)
 	{
 	  int within_current_scope;
 	  if (b->exp_valid_block != NULL)
@@ -848,7 +928,7 @@ which its expression is valid.\n", b->number);
 	  else
 	    {
 	      /* We will stop here */
-	      if (b->enable == temporary)
+	      if (b->disposition == disable)
 		b->enable = disabled;
 	      bs->commands = b->commands;
 	      if (b->silent)
@@ -895,12 +975,16 @@ which its expression is valid.\n", b->number);
   return bs;
 }
 
+/* Nonzero if we should step constantly (e.g. watchpoints on machines
+   without hardware support).  This isn't related to a specific bpstat,
+   just to things like whether watchpoints are set.  */
+
 int 
 bpstat_should_step ()
 {
   struct breakpoint *b;
   ALL_BREAKPOINTS (b)
-    if (b->enable != disabled && b->exp != NULL)
+    if (b->enable == enabled && b->type == bp_watchpoint)
       return 1;
   return 0;
 }
@@ -910,50 +994,57 @@ bpstat_should_step ()
    is nonzero, process only watchpoints.  */
 
 static void
-breakpoint_1 (bnum, watchpoints)
+breakpoint_1 (bnum, allflag)
      int bnum;
-     int watchpoints;
+     int allflag;
 {
   register struct breakpoint *b;
   register struct command_line *l;
   register struct symbol *sym;
   CORE_ADDR last_addr = (CORE_ADDR)-1;
-  int header_printed = 0;
+  int found_a_breakpoint = 0;
+  static char *bptypes[] = {"breakpoint", "until", "finish", "watchpoint",
+			      "longjmp", "longjmp resume"};
+  static char *bpdisps[] = {"del", "dis", "keep"};
+  static char bpenables[] = "ny";
+
+  if (!breakpoint_chain)
+    {
+      printf_filtered ("No breakpoints or watchpoints.\n");
+      return;
+    }
   
   ALL_BREAKPOINTS (b)
-    if (bnum == -1 || bnum == b->number)
+    if (bnum == -1
+	|| bnum == b->number)
       {
-	if (b->address == 0 && !watchpoints)
-	  {
-	    if (bnum == -1)
-	      continue;
-	    error ("That is a watchpoint, not a breakpoint.");
-	  }
-	if (b->address != 0 && watchpoints)
-	  {
-	    if (bnum == -1)
-	      continue;
-	    error ("That is a breakpoint, not a watchpoint.");
-	  }
+/*  We only print out user settable breakpoints unless the allflag is set. */
+	if (!allflag
+	    && b->type != bp_breakpoint
+	    && b->type != bp_watchpoint)
+	  continue;
 
-	if (!header_printed)
-	  {
-	    if (watchpoints)
-	      printf_filtered ("    Enb   Expression\n");
-	    else if (addressprint)
-	      printf_filtered ("    Enb   Address    Where\n");
-	    else
-	      printf_filtered ("    Enb   Where\n");
-	    header_printed = 1;
-	  }
+	if (!found_a_breakpoint++)
+	  printf_filtered ("Num Type           Disp Enb %sWhat\n",
+			   addressprint ? "Address    " : "");
 
-	printf_filtered ("#%-3d %c ", b->number, "nyod"[(int) b->enable]);
-	if (b->address == 0) {
-	  printf_filtered (" ");
-	  print_expression (b->exp, stdout);
-	} else {
+	printf_filtered ("%-3d %-14s %-4s %-3c ",
+			 b->number,
+			 bptypes[(int)b->type],
+			 bpdisps[(int)b->disposition],
+			 bpenables[(int)b->enable]);
+	switch (b->type)
+	  {
+	  case bp_watchpoint:
+	    print_expression (b->exp, stdout);
+	    break;
+	  case bp_breakpoint:
+	  case bp_until:
+	  case bp_finish:
+	  case bp_longjmp:
+	  case bp_longjmp_resume:
 	    if (addressprint)
-	      printf_filtered (" %s ", local_hex_string_custom(b->address, "08"));
+	      printf_filtered ("%s ", local_hex_string_custom(b->address, "08"));
 
 	    last_addr = b->address;
 	    if (b->symtab)
@@ -961,7 +1052,7 @@ breakpoint_1 (bnum, watchpoints)
 		sym = find_pc_function (b->address);
 		if (sym)
 		  {
-		    fputs_filtered (" in ", stdout);
+		    fputs_filtered ("in ", stdout);
 		    fputs_demangled (SYMBOL_NAME (sym), stdout, 1);
 		    fputs_filtered (" at ", stdout);
 		  }
@@ -995,19 +1086,14 @@ breakpoint_1 (bnum, watchpoints)
 	    }
       }
 
-  if (!header_printed)
-    {
-      char *which = watchpoints ? "watch" : "break";
-      if (bnum == -1)
-	printf_filtered ("No %spoints.\n", which);
-      else
-	printf_filtered ("No %spoint numbered %d.\n", which, bnum);
-    }
-
-  /* Compare against (CORE_ADDR)-1 in case some compiler decides
-     that a comparison of an unsigned with -1 is always false.  */
-  if (last_addr != (CORE_ADDR)-1)
-    set_next_address (last_addr);
+  if (!found_a_breakpoint
+      && bnum != -1)
+    printf_filtered ("No breakpoint or watchpoint number %d.\n", bnum);
+  else
+    /* Compare against (CORE_ADDR)-1 in case some compiler decides
+       that a comparison of an unsigned with -1 is always false.  */
+    if (last_addr != (CORE_ADDR)-1)
+      set_next_address (last_addr);
 }
 
 /* ARGSUSED */
@@ -1026,7 +1112,7 @@ breakpoints_info (bnum_exp, from_tty)
 
 /* ARGSUSED */
 static void
-watchpoints_info (bnum_exp, from_tty)
+all_breakpoints_info (bnum_exp, from_tty)
      char *bnum_exp;
      int from_tty;
 {
@@ -1151,46 +1237,139 @@ set_raw_breakpoint (sal)
   return b;
 }
 
+static void
+create_longjmp_breakpoint(func_name)
+     char *func_name;
+{
+  struct symtab_and_line sal;
+  struct breakpoint *b;
+  static int internal_breakpoint_number = -1;
+
+  if (func_name != NULL)
+    {
+      struct minimal_symbol *m;
+
+      m = lookup_minimal_symbol(func_name, (struct objfile *)NULL);
+      if (m)
+	sal.pc = m->address;
+      else
+	return;
+    }
+  else
+    sal.pc = 0;
+
+  sal.symtab = NULL;
+  sal.line = 0;
+
+  b = set_raw_breakpoint(sal);
+  if (!b) return;
+
+  b->type = func_name != NULL ? bp_longjmp : bp_longjmp_resume;
+  b->disposition = donttouch;
+  b->enable = disabled;
+  b->silent = 1;
+  if (func_name)
+    b->addr_string = strsave(func_name);
+  b->number = internal_breakpoint_number--;
+}
+
+/* Call this routine when stepping and nexting to enable a breakpoint if we do
+   a longjmp().  When we hit that breakpoint, call
+   set_longjmp_resume_breakpoint() to figure out where we are going. */
+
+void
+enable_longjmp_breakpoint()
+{
+  register struct breakpoint *b;
+
+  ALL_BREAKPOINTS (b)
+    if (b->type == bp_longjmp)
+      b->enable = enabled;
+}
+
+void
+disable_longjmp_breakpoint()
+{
+  register struct breakpoint *b;
+
+  ALL_BREAKPOINTS (b)
+    if (b->type == bp_longjmp
+	|| b->type == bp_longjmp_resume)
+      b->enable = disabled;
+}
+
+/* Call this after hitting the longjmp() breakpoint.  Use this to set a new
+   breakpoint at the target of the jmp_buf.
+
+   FIXME - This ought to be done by setting a temporary breakpoint that gets
+   deleted automatically...
+*/
+
+void
+set_longjmp_resume_breakpoint(pc, frame)
+     CORE_ADDR pc;
+     FRAME frame;
+{
+  register struct breakpoint *b;
+
+  ALL_BREAKPOINTS (b)
+    if (b->type == bp_longjmp_resume)
+      {
+	b->address = pc;
+	b->enable = enabled;
+	if (frame != NULL)
+	  b->frame = FRAME_FP(frame);
+	else
+	  b->frame = 0;
+	return;
+      }
+}
+
 /* Set a breakpoint that will evaporate an end of command
    at address specified by SAL.
    Restrict it to frame FRAME if FRAME is nonzero.  */
 
-void
-set_momentary_breakpoint (sal, frame)
+struct breakpoint *
+set_momentary_breakpoint (sal, frame, type)
      struct symtab_and_line sal;
      FRAME frame;
+     enum bptype type;
 {
   register struct breakpoint *b;
   b = set_raw_breakpoint (sal);
-  b->number = -3;
-  b->enable = delete;
+  b->type = type;
+  b->enable = enabled;
+  b->disposition = donttouch;
   b->frame = (frame ? FRAME_FP (frame) : 0);
+  return b;
 }
 
+#if 0
 void
 clear_momentary_breakpoints ()
 {
   register struct breakpoint *b;
   ALL_BREAKPOINTS (b)
-    if (b->number == -3)
+    if (b->disposition == delete)
       {
 	delete_breakpoint (b);
 	break;
       }
 }
+#endif
 
 /* Tell the user we have just set a breakpoint B.  */
 static void
 mention (b)
      struct breakpoint *b;
 {
-  if (b->exp)
+  switch (b->type)
     {
+    case bp_watchpoint:
       printf_filtered ("Watchpoint %d: ", b->number);
       print_expression (b->exp, stdout);
-    }
-  else
-    {
+      break;
+    case bp_breakpoint:
       printf_filtered ("Breakpoint %d at %s", b->number,
 		       local_hex_string(b->address));
       if (b->symtab)
@@ -1228,14 +1407,15 @@ set_breakpoint (s, line, tempflag, addr_string)
   b = set_raw_breakpoint (sal);
   set_breakpoint_count (breakpoint_count + 1);
   b->number = breakpoint_count;
+  b->type = bp_breakpoint;
   b->cond = 0;
   b->addr_string = addr_string;
-  if (tempflag)
-    b->enable = temporary;
+  b->enable = enabled;
+  b->disposition = tempflag ? delete : donttouch;
 
   mention (b);
 }
-#endif
+#endif /* 0 */
 
 /* Set a breakpoint according to ARG (function, linenum or *address)
    and make it temporary if TEMPFLAG is nonzero. */
@@ -1338,6 +1518,7 @@ break_command_1 (arg, tempflag, from_tty)
       b = set_raw_breakpoint (sal);
       set_breakpoint_count (breakpoint_count + 1);
       b->number = breakpoint_count;
+      b->type = bp_breakpoint;
       b->cond = cond;
       
       if (addr_start)
@@ -1345,8 +1526,8 @@ break_command_1 (arg, tempflag, from_tty)
       if (cond_start)
 	b->cond_string = savestring (cond_start, cond_end - cond_start);
 				     
-      if (tempflag)
-	b->enable = temporary;
+      b->enable = enabled;
+      b->disposition = tempflag ? delete : donttouch;
 
       mention (b);
     }
@@ -1356,7 +1537,7 @@ break_command_1 (arg, tempflag, from_tty)
       printf ("Multiple breakpoints were set.\n");
       printf ("Use the \"delete\" command to delete unwanted breakpoints.\n");
     }
-  free (sals.sals);
+  free ((PTR)sals.sals);
 }
 
 /* Helper function for break_command_1 and disassemble_command.  */
@@ -1420,6 +1601,8 @@ watch_command (arg, from_tty)
   b = set_raw_breakpoint (sal);
   set_breakpoint_count (breakpoint_count + 1);
   b->number = breakpoint_count;
+  b->type = bp_watchpoint;
+  b->disposition = donttouch;
   b->exp = exp;
   b->exp_valid_block = exp_valid_block;
   b->val = val;
@@ -1441,6 +1624,8 @@ until_break_command (arg, from_tty)
   struct symtabs_and_lines sals;
   struct symtab_and_line sal;
   FRAME prev_frame = get_prev_frame (selected_frame);
+  struct breakpoint *breakpoint;
+  struct cleanup *old_chain;
 
   clear_proceed_status ();
 
@@ -1457,14 +1642,17 @@ until_break_command (arg, from_tty)
     error ("Couldn't get information on specified line.");
   
   sal = sals.sals[0];
-  free (sals.sals);		/* malloc'd, so freed */
+  free ((PTR)sals.sals);		/* malloc'd, so freed */
   
   if (*arg)
     error ("Junk at end of arguments.");
   
   resolve_sal_pc (&sal);
-  set_momentary_breakpoint (sal, selected_frame);
   
+  breakpoint = set_momentary_breakpoint (sal, selected_frame, bp_until);
+  
+  old_chain = make_cleanup(delete_breakpoint, breakpoint);
+
   /* Keep within the current frame */
   
   if (prev_frame)
@@ -1474,10 +1662,12 @@ until_break_command (arg, from_tty)
       fi = get_frame_info (prev_frame);
       sal = find_pc_line (fi->pc, 0);
       sal.pc = fi->pc;
-      set_momentary_breakpoint (sal, prev_frame);
+      breakpoint = set_momentary_breakpoint (sal, prev_frame, bp_until);
+      make_cleanup(delete_breakpoint, breakpoint);
     }
   
   proceed (-1, -1, 0);
+  do_cleanups(old_chain);
 }
 
 #if 0
@@ -1579,7 +1769,6 @@ static struct symtabs_and_lines
 get_catch_sals (this_level_only)
      int this_level_only;
 {
-  extern struct blockvector *blockvector_for_pc ();
   register struct blockvector *bl;
   register struct block *block;
   int index, have_default = 0;
@@ -1692,7 +1881,7 @@ get_catch_sals (this_level_only)
 
 /* Commands to deal with catching exceptions.  */
 
-void
+static void
 catch_command_1 (arg, tempflag, from_tty)
      char *arg;
      int tempflag;
@@ -1707,7 +1896,6 @@ catch_command_1 (arg, tempflag, from_tty)
   register struct breakpoint *b;
   char *save_arg;
   int i;
-  CORE_ADDR pc;
 
   sal.line = sal.pc = sal.end = 0;
   sal.symtab = 0;
@@ -1759,10 +1947,12 @@ catch_command_1 (arg, tempflag, from_tty)
 	describe_other_breakpoints (sal.pc);
 
       b = set_raw_breakpoint (sal);
-      b->number = ++breakpoint_count;
+      set_breakpoint_count (breakpoint_count + 1);
+      b->number = breakpoint_count;
+      b->type = bp_breakpoint;
       b->cond = cond;
-      if (tempflag)
-	b->enable = temporary;
+      b->enable = enabled;
+      b->disposition = tempflag ? delete : donttouch;
 
       printf ("Breakpoint %d at %s", b->number, local_hex_string(b->address));
       if (b->symtab)
@@ -1775,7 +1965,7 @@ catch_command_1 (arg, tempflag, from_tty)
       printf ("Multiple breakpoints were set.\n");
       printf ("Use the \"delete\" command to delete unwanted breakpoints.\n");
     }
-  free (sals.sals);
+  free ((PTR)sals.sals);
 }
 
 #if 0
@@ -1860,7 +2050,7 @@ clear_command (arg, from_tty)
 
       ALL_BREAKPOINTS (b)
 	while (b->next
-	       && b->next->address != 0
+	       && b->next->type != bp_watchpoint
 	       && (sal.pc ? b->next->address == sal.pc
 		   : (b->next->symtab == sal.symtab
 		      && b->next->line_number == sal.line)))
@@ -1890,7 +2080,7 @@ clear_command (arg, from_tty)
 	}
       if (from_tty) putchar ('\n');
     }
-  free (sals.sals);
+  free ((PTR)sals.sals);
 }
 
 /* Delete breakpoint in BS if they are `delete' breakpoints.
@@ -1901,13 +2091,13 @@ breakpoint_auto_delete (bs)
      bpstat bs;
 {
   for (; bs; bs = bs->next)
-    if (bs->breakpoint_at && bs->breakpoint_at->enable == delete)
+    if (bs->breakpoint_at && bs->breakpoint_at->disposition == delete)
       delete_breakpoint (bs->breakpoint_at);
 }
 
 /* Delete a breakpoint and clean up all traces of it in the data structures. */
 
-static void
+void
 delete_breakpoint (bpt)
      struct breakpoint *bpt;
 {
@@ -1931,13 +2121,13 @@ delete_breakpoint (bpt)
 
   free_command_lines (&bpt->commands);
   if (bpt->cond)
-    free (bpt->cond);
+    free ((PTR)bpt->cond);
   if (bpt->cond_string != NULL)
-    free (bpt->cond_string);
+    free ((PTR)bpt->cond_string);
   if (bpt->addr_string != NULL)
-    free (bpt->addr_string);
+    free ((PTR)bpt->addr_string);
 
-  if (xgdb_verbose && bpt->number >=0)
+  if (xgdb_verbose && bpt->type == bp_breakpoint)
     printf ("breakpoint #%d deleted\n", bpt->number);
 
   /* Be sure no bpstat's are pointing at it after it's been freed.  */
@@ -1945,11 +2135,9 @@ delete_breakpoint (bpt)
   for (bs = stop_bpstat; bs; bs = bs->next)
     if (bs->breakpoint_at == bpt)
       bs->breakpoint_at = NULL;
-  free (bpt);
+  free ((PTR)bpt);
 }
 
-static void map_breakpoint_numbers ();
-
 static void
 delete_command (arg, from_tty)
      char *arg;
@@ -1985,8 +2173,15 @@ breakpoint_re_set_one (bint)
   char *s;
   enum enable save_enable;
 
-  if (b->address != 0 && b->addr_string != NULL)
+  switch (b->type)
     {
+    case bp_breakpoint:
+      if (b->addr_string == NULL)
+	{
+	  /* Anything without a string can't be re-set. */
+	  delete_breakpoint (b);
+	  return 0;
+	}
       /* In case we have a problem, disable this breakpoint.  We'll restore
 	 its status if we succeed.  */
       save_enable = b->enable;
@@ -1997,28 +2192,42 @@ breakpoint_re_set_one (bint)
       for (i = 0; i < sals.nelts; i++)
 	{
 	  resolve_sal_pc (&sals.sals[i]);
-	  b->symtab = sals.sals[i].symtab;
-	  b->line_number = sals.sals[i].line;
-	  b->address = sals.sals[i].pc;
-
-	  if (b->cond_string != NULL)
+	  if (b->symtab != sals.sals[i].symtab
+	      || b->line_number != sals.sals[i].line
+	      || b->address != sals.sals[i].pc)
 	    {
-	      s = b->cond_string;
-	      b->cond = parse_exp_1 (&s, block_for_pc (sals.sals[i].pc), 0);
-	    }
-	  
-	  check_duplicates (b->address);
+	      b->symtab = sals.sals[i].symtab;
+	      b->line_number = sals.sals[i].line;
+	      b->address = sals.sals[i].pc;
 
+	      if (b->cond_string != NULL)
+		{
+		  s = b->cond_string;
+		  b->cond = parse_exp_1 (&s, block_for_pc (sals.sals[i].pc), 0);
+		}
+	  
+	      check_duplicates (b->address);
+
+	      mention (b);
+	    }
 	  b->enable = save_enable;	/* Restore it, this worked. */
-	  mention (b);
 	}
-      free (sals.sals);
-    }
-  else
-    {
-      /* Anything without a string can't be re-set. */
+      free ((PTR)sals.sals);
+      break;
+    case bp_watchpoint:
+      /* FIXME!  This is the wrong thing to do.... */
       delete_breakpoint (b);
+      break;
+    default:
+      printf_filtered ("Deleting unknown breakpoint type %d\n", b->type);
+    case bp_until:
+    case bp_finish:
+    case bp_longjmp:
+    case bp_longjmp_resume:
+      delete_breakpoint (b);
+      break;
     }
+
   return 0;
 }
 
@@ -2026,19 +2235,27 @@ breakpoint_re_set_one (bint)
 void
 breakpoint_re_set ()
 {
-  struct breakpoint *b;
+  struct breakpoint *b, *temp;
   static char message1[] = "Error in re-setting breakpoint %d:\n";
   char message[sizeof (message1) + 30 /* slop */];
   
-  ALL_BREAKPOINTS (b)
+  ALL_BREAKPOINTS_SAFE (b, temp)
     {
-      b->symtab = 0;		/* Be sure we don't point to old dead symtab */
       sprintf (message, message1, b->number);	/* Format possible error msg */
       (void) catch_errors (breakpoint_re_set_one, (char *) b, message);
     }
 
+  create_longjmp_breakpoint("longjmp");
+  create_longjmp_breakpoint("_longjmp");
+  create_longjmp_breakpoint("siglongjmp");
+  create_longjmp_breakpoint(NULL);
+
+#if 0
+  /* Took this out (temporaliy at least), since it produces an extra 
+     blank line at startup. This messes up the gdbtests. -PB */
   /* Blank line to finish off all those mention() messages we just printed.  */
   printf_filtered ("\n");
+#endif
 }
 
 /* Set ignore-count of breakpoint number BPTNUM to COUNT.
@@ -2113,7 +2330,7 @@ ignore_command (args, from_tty)
 static void
 map_breakpoint_numbers (args, function)
      char *args;
-     void (*function) ();
+     void (*function) PARAMS ((struct breakpoint *));
 {
   register char *p = args;
   char *p1;
@@ -2147,11 +2364,11 @@ enable_breakpoint (bpt)
 {
   bpt->enable = enabled;
 
-  if (xgdb_verbose && bpt->number >= 0)
+  if (xgdb_verbose && bpt->type == bp_breakpoint)
     printf ("breakpoint #%d enabled\n", bpt->number);
 
   check_duplicates (bpt->address);
-  if (bpt->val != NULL)
+  if (bpt->type == bp_watchpoint)
     {
       if (bpt->exp_valid_block != NULL
        && !contained_in (get_selected_block (), bpt->exp_valid_block))
@@ -2189,7 +2406,7 @@ disable_breakpoint (bpt)
 {
   bpt->enable = disabled;
 
-  if (xgdb_verbose && bpt->number >= 0)
+  if (xgdb_verbose && bpt->type == bp_breakpoint)
     printf ("breakpoint #%d disabled\n", bpt->number);
 
   check_duplicates (bpt->address);
@@ -2213,7 +2430,8 @@ static void
 enable_once_breakpoint (bpt)
      struct breakpoint *bpt;
 {
-  bpt->enable = temporary;
+  bpt->enable = enabled;
+  bpt->disposition = disable;
 
   check_duplicates (bpt->address);
 }
@@ -2231,7 +2449,8 @@ static void
 enable_delete_breakpoint (bpt)
      struct breakpoint *bpt;
 {
-  bpt->enable = delete;
+  bpt->enable = enabled;
+  bpt->disposition = delete;
 
   check_duplicates (bpt->address);
 }
@@ -2406,10 +2625,32 @@ Do \"help breakpoints\" for info on other commands dealing with breakpoints.");
   add_com_alias ("brea", "break", class_run, 1);
 
   add_info ("breakpoints", breakpoints_info,
+	    "Status of user-settable breakpoints, or breakpoint number NUMBER.\n\
+The \"Type\" column indicates one of:\n\
+\tbreakpoint     - for normal breakpoints\n\
+\twatchpoint     - for watchpoints\n\
+The \"Disp\" column contains one of \"keep\", \"del\", or \"dis\" to indicate\n\
+the disposition of the breakpoint after it gets hit.  \"dis\" means that the\n\
+breakpoint will be disabled.  The \"Address\" and \"What\" columns indicate the\n\
+address and file/line number respectively.\n\n\
+Convenience variable \"$_\" and default examine address for \"x\"\n\
+are set to the address of the last breakpoint listed.\n\n\
+Convenience variable \"$bpnum\" contains the number of the last\n\
+breakpoint set.");
+
+  add_info ("all-breakpoints", all_breakpoints_info,
 	    "Status of all breakpoints, or breakpoint number NUMBER.\n\
-Second column is \"y\" for enabled breakpoint, \"n\" for disabled,\n\
-\"o\" for enabled once (disable when hit), \"d\" for enable but delete when hit.\n\
-Then come the address and the file/line number.\n\n\
+The \"Type\" column indicates one of:\n\
+\tbreakpoint     - for normal breakpoints\n\
+\twatchpoint     - for watchpoints\n\
+\tlongjmp        - for internal breakpoints to handle stepping through longjmp()\n\
+\tlongjmp resume - for internal breakpoints at the target of longjmp()\n\
+\tuntil          - for internal breakpoints used by the \"until\" command\n\
+\tfinish         - for internal breakpoints used by the \"finish\" command\n\
+The \"Disp\" column contains one of \"keep\", \"del\", or \"dis\" to indicate\n\
+the disposition of the breakpoint after it gets hit.  \"dis\" means that the\n\
+breakpoint will be disabled.  The \"Address\" and \"What\" columns indicate the\n\
+address and file/line number respectively.\n\n\
 Convenience variable \"$_\" and default examine address for \"x\"\n\
 are set to the address of the last breakpoint listed.\n\n\
 Convenience variable \"$bpnum\" contains the number of the last\n\
@@ -2432,7 +2673,38 @@ Do \"help breakpoints\" for info on other commands dealing with breakpoints.");
 A watchpoint stops execution of your program whenever the value of\n\
 an expression changes.");
 
-  add_info ("watchpoints", watchpoints_info,
-	    "Status of all watchpoints, or watchpoint number NUMBER.\n\
-Second column is \"y\" for enabled watchpoints, \"n\" for disabled.");
+  add_info ("watchpoints", breakpoints_info,
+	    "Synonym for ``info breakpoints''.");
 }
+
+#ifdef IBM6000_HOST
+/* Where should this function go? It is used by AIX only. FIXME. */
+
+/* Breakpoint address relocation used to be done in breakpoint_re_set(). That
+   approach the following problem:
+
+     before running the program, if a file is list, then a breakpoint is
+     set (just the line number), then if we switch into another file and run
+     the program, just a line number as a breakpoint address was not
+     descriptive enough and breakpoint was ending up in a different file's
+     similar line. 
+
+  I don't think any other platform has this breakpoint relocation problem, so this
+  is not an issue for other platforms. */
+   
+void
+fixup_breakpoints (low, high, delta)
+  CORE_ADDR low;
+  CORE_ADDR high;
+  CORE_ADDR delta;
+{
+  struct breakpoint *b;
+  extern struct breakpoint *breakpoint_chain;
+
+  ALL_BREAKPOINTS (b)
+    {
+     if (b->address >= low && b->address <= high)
+       b->address += delta;
+    }
+}
+#endif
