@@ -1,5 +1,5 @@
 /* Memory-access and commands for "inferior" (child) process, for GDB.
-   Copyright 1986, 1987, 1988, 1989, 1991 Free Software Foundation, Inc.
+   Copyright 1986, 1987, 1988, 1989, 1991, 1992 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -17,12 +17,12 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-#include <stdio.h>
+#include "defs.h"
 #include <signal.h>
 #include <sys/param.h>
 #include <string.h>
-#include "defs.h"
 #include "symtab.h"
+#include "gdbtypes.h"
 #include "frame.h"
 #include "inferior.h"
 #include "environ.h"
@@ -31,9 +31,82 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "gdbcore.h"
 #include "target.h"
 
-extern char *sys_siglist[];
+static void
+continue_command PARAMS ((char *, int));
 
-extern void until_break_command ();	/* breakpoint.c */
+static void
+until_next_command PARAMS ((int));
+
+static void 
+until_command PARAMS ((char *, int));
+
+static void
+path_info PARAMS ((char *, int));
+
+static void
+path_command PARAMS ((char *, int));
+
+static void
+unset_command PARAMS ((char *, int));
+
+static void
+float_info PARAMS ((char *, int));
+
+static void
+detach_command PARAMS ((char *, int));
+
+static void
+nofp_registers_info PARAMS ((char *, int));
+
+static void
+all_registers_info PARAMS ((char *, int));
+
+static void
+registers_info PARAMS ((char *, int));
+
+static void
+do_registers_info PARAMS ((int, int));
+
+static void
+unset_environment_command PARAMS ((char *, int));
+
+static void
+set_environment_command PARAMS ((char *, int));
+
+static void
+environment_info PARAMS ((char *, int));
+
+static void
+program_info PARAMS ((char *, int));
+
+static void
+finish_command PARAMS ((char *, int));
+
+static void
+signal_command PARAMS ((char *, int));
+
+static void
+jump_command PARAMS ((char *, int));
+
+static void
+step_1 PARAMS ((int, int, char *));
+
+static void
+nexti_command PARAMS ((char *, int));
+
+static void
+stepi_command PARAMS ((char *, int));
+
+static void
+next_command PARAMS ((char *, int));
+
+static void
+step_command PARAMS ((char *, int));
+
+static void
+run_command PARAMS ((char *, int));
+
+extern char *sys_siglist[];
 
 #define ERROR_NO_INFERIOR \
    if (!target_has_execution) error ("The program is not being run.");
@@ -117,9 +190,6 @@ int step_multi;
 
 struct environ *inferior_environ;
 
-CORE_ADDR read_pc ();
-void breakpoint_clear_ignore_counts ();
-
 
 /* ARGSUSED */
 void
@@ -148,12 +218,12 @@ run_command (args, from_tty)
 	  !query ("The program being debugged has been started already.\n\
 Start it from the beginning? "))
 	error ("Program not restarted.");
-      target_kill ((char *)0, 0);
+      target_kill ();
     }
 
   exec_file = (char *) get_exec_file (0);
 
-  /* The exec file is re-read every time we do an inferior_died, so
+  /* The exec file is re-read every time we do a generic_mourn_inferior, so
      we just have to worry about the symbol file.  */
   reread_symbols ();
 
@@ -176,7 +246,7 @@ Start it from the beginning? "))
 			  environ_vector (inferior_environ));
 }
 
-void
+static void
 continue_command (proc_count_exp, from_tty)
      char *proc_count_exp;
      int from_tty;
@@ -216,12 +286,11 @@ continue_command (proc_count_exp, from_tty)
 }
 
 /* Step until outside of current statement.  */
-static void step_1 ();
 
 /* ARGSUSED */
 static void
 step_command (count_string, from_tty)
-     char * count_string;
+     char *count_string;
      int from_tty;
 {
   step_1 (0, 0, count_string);
@@ -232,7 +301,7 @@ step_command (count_string, from_tty)
 /* ARGSUSED */
 static void
 next_command (count_string, from_tty)
-     char * count_string;
+     char *count_string;
      int from_tty;
 {
   step_1 (1, 0, count_string);
@@ -243,7 +312,7 @@ next_command (count_string, from_tty)
 /* ARGSUSED */
 static void
 stepi_command (count_string, from_tty)
-     char * count_string;
+     char *count_string;
      int from_tty;
 {
   step_1 (0, 1, count_string);
@@ -252,7 +321,7 @@ stepi_command (count_string, from_tty)
 /* ARGSUSED */
 static void
 nexti_command (count_string, from_tty)
-     char * count_string;
+     char *count_string;
      int from_tty;
 {
   step_1 (1, 1, count_string);
@@ -266,15 +335,21 @@ step_1 (skip_subroutines, single_inst, count_string)
 {
   register int count = 1;
   FRAME fr;
+  struct cleanup *cleanups = 0;
 
   ERROR_NO_INFERIOR;
   count = count_string ? parse_and_eval_address (count_string) : 1;
 
+  if (!single_inst || skip_subroutines) /* leave si command alone */
+    {
+      enable_longjmp_breakpoint();
+      cleanups = make_cleanup(disable_longjmp_breakpoint, 0);
+    }
+
   for (; count > 0; count--)
     {
       clear_proceed_status ();
 
-
       fr = get_current_frame ();
       if (!fr)				/* Avoid coredump here.  Why tho? */
 	error ("No current frame");
@@ -285,22 +360,22 @@ step_1 (skip_subroutines, single_inst, count_string)
 	  find_pc_line_pc_range (stop_pc, &step_range_start, &step_range_end);
 	  if (step_range_end == 0)
 	    {
-	      int misc;
+	      struct minimal_symbol *msymbol;
 
-	      misc = find_pc_misc_function (stop_pc);
+	      msymbol = lookup_minimal_symbol_by_pc (stop_pc);
 	      target_terminal_ours ();
 	      printf ("Current function has no line number information.\n");
 	      fflush (stdout);
 
 	      /* No info or after _etext ("Can't happen") */
-	      if (misc == -1 || misc == misc_function_count - 1)
+	      if (msymbol == NULL || (msymbol + 1) -> name == NULL)
 		error ("No data available on pc function.");
 
 	      printf ("Single stepping until function exit.\n");
 	      fflush (stdout);
 
-	      step_range_start = misc_function_vector[misc].address;
-	      step_range_end = misc_function_vector[misc + 1].address;
+	      step_range_start = msymbol -> address;
+	      step_range_end = (msymbol + 1) -> address;
 	    }
 	}
       else
@@ -325,6 +400,9 @@ step_1 (skip_subroutines, single_inst, count_string)
       write_register (NPC_REGNUM, read_register (PC_REGNUM));
 #endif
     }
+
+  if (!single_inst || skip_subroutines)
+    do_cleanups(cleanups);
 }
 
 /* Continue program at specified address.  */
@@ -350,7 +428,7 @@ jump_command (arg, from_tty)
     }
 
   sal = sals.sals[0];
-  free (sals.sals);
+  free ((PTR)sals.sals);
 
   if (sal.symtab == 0 && sal.pc == 0)
     error ("No source file has been specified.");
@@ -454,7 +532,7 @@ The expression which contained the function call has been discarded.");
    of wait_for_inferior and the proceed status code. -- randy */
 
 /* ARGSUSED */
-void
+static void
 until_next_command (from_tty)
      int from_tty;
 {
@@ -476,12 +554,12 @@ until_next_command (from_tty)
   
   if (!func)
     {
-      int misc_func = find_pc_misc_function (pc);
+      struct minimal_symbol *msymbol = lookup_minimal_symbol_by_pc (pc);
       
-      if (misc_func != -1)
+      if (msymbol == NULL)
 	error ("Execution is not within a known function.");
       
-      step_range_start = misc_function_vector[misc_func].address;
+      step_range_start = msymbol -> address;
       step_range_end = pc;
     }
   else
@@ -500,7 +578,7 @@ until_next_command (from_tty)
   proceed ((CORE_ADDR) -1, -1, 1);
 }
 
-void 
+static void 
 until_command (arg, from_tty)
      char *arg;
      int from_tty;
@@ -525,6 +603,8 @@ finish_command (arg, from_tty)
   register FRAME frame;
   struct frame_info *fi;
   register struct symbol *function;
+  struct breakpoint *breakpoint;
+  struct cleanup *old_chain;
 
   if (arg)
     error ("The \"finish\" command does not take any arguments.");
@@ -542,7 +622,10 @@ finish_command (arg, from_tty)
   fi = get_frame_info (frame);
   sal = find_pc_line (fi->pc, 0);
   sal.pc = fi->pc;
-  set_momentary_breakpoint (sal, frame);
+
+  breakpoint = set_momentary_breakpoint (sal, frame, bp_finish);
+
+  old_chain = make_cleanup(delete_breakpoint, breakpoint);
 
   /* Find the function we will return from.  */
 
@@ -560,7 +643,9 @@ finish_command (arg, from_tty)
   proceed_to_finish = 1;		/* We want stop_registers, please... */
   proceed ((CORE_ADDR) -1, -1, 0);
 
-  if (bpstat_momentary_breakpoint (stop_bpstat) && function != 0)
+  /* Did we stop at our breakpoint? */
+  if (bpstat_find_breakpoint(stop_bpstat, breakpoint) != NULL
+      && function != 0)
     {
       struct type *value_type;
       register value val;
@@ -585,6 +670,7 @@ finish_command (arg, from_tty)
       value_print (val, stdout, 0, Val_no_prettyprint);
       printf_filtered ("\n");
     }
+  do_cleanups(old_chain);
 }
 
 /* ARGSUSED */
@@ -634,8 +720,9 @@ program_info (args, from_tty)
 }
 
 static void
-environment_info (var)
+environment_info (var, from_tty)
      char *var;
+     int from_tty;
 {
   if (var)
     {
@@ -654,8 +741,9 @@ environment_info (var)
 }
 
 static void
-set_environment_command (arg)
+set_environment_command (arg, from_tty)
      char *arg;
+     int from_tty;
 {
   register char *p, *val, *var;
   int nullset = 0;
@@ -737,7 +825,7 @@ unset_environment_command (var, from_tty)
 const static char path_var_name[] = "PATH";
 
 /* ARGSUSED */
-void
+static void
 path_info (args, from_tty)
      char *args;
      int from_tty;
@@ -748,7 +836,7 @@ path_info (args, from_tty)
 
 /* Add zero or more directories to the front of the execution path.  */
 
-void
+static void
 path_command (dirname, from_tty)
      char *dirname;
      int from_tty;
@@ -909,15 +997,17 @@ registers_info (addr_exp, fpregs)
 }
 
 static void
-all_registers_info (addr_exp)
+all_registers_info (addr_exp, from_tty)
      char *addr_exp;
+     int from_tty;
 {
   registers_info (addr_exp, 1);
 }
 
 static void
-nofp_registers_info (addr_exp)
+nofp_registers_info (addr_exp, from_tty)
      char *addr_exp;
+     int from_tty;
 {
   registers_info (addr_exp, 0);
 }
@@ -970,8 +1060,9 @@ detach_command (args, from_tty)
 
 /* ARGSUSED */
 static void
-float_info (addr_exp)
+float_info (addr_exp, from_tty)
      char *addr_exp;
+     int from_tty;
 {
 #ifdef FLOAT_INFO
 	FLOAT_INFO;
