@@ -1,5 +1,6 @@
 /* Shared code to pre-read a stab (dbx-style), when building a psymtab.
-   Copyright (C) 1986-1991 Free Software Foundation, Inc.
+   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992 Free Software Foundation,
+   Inc.
 
 This file is part of GDB.
 
@@ -47,8 +48,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	  SET_NAMESTRING();
 
 	bss_ext_symbol:
-	  record_misc_function (namestring, CUR_SYMBOL_VALUE,
-				CUR_SYMBOL_TYPE); /* Always */
+	  record_minimal_symbol (namestring, CUR_SYMBOL_VALUE,
+				 CUR_SYMBOL_TYPE, objfile); /* Always */
 #endif /* DBXREAD_ONLY */
 	  continue;
 
@@ -71,12 +72,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	      || (namestring [(nsl = strlen (namestring)) - 1] == 'o'
 		  && namestring [nsl - 2] == '.'))
 	    {
-	      if (entry_point < CUR_SYMBOL_VALUE
-		  && entry_point >= last_o_file_start
-		  && addr == 0)		/* FIXME nogood nomore */
+	      if (objfile -> ei.entry_point <  CUR_SYMBOL_VALUE &&
+		  objfile -> ei.entry_point >= last_o_file_start &&
+		  addr == 0)		/* FIXME nogood nomore */
 		{
-		  startup_file_start = last_o_file_start;
-		  startup_file_end = CUR_SYMBOL_VALUE;
+		  objfile -> ei.entry_file_lowpc = last_o_file_start;
+		  objfile -> ei.entry_file_highpc = CUR_SYMBOL_VALUE;
 		}
 	      if (past_first_source_file && pst
 		  /* The gould NP1 uses low values for .o and -l symbols
@@ -108,8 +109,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	      || VTBL_PREFIX_P ((namestring+HASH_OFFSET)))
 	    {
 	      /* Not really a function here, but... */
-	      record_misc_function (namestring, CUR_SYMBOL_VALUE,
-				    CUR_SYMBOL_TYPE); /* Always */
+	      record_minimal_symbol (namestring, CUR_SYMBOL_VALUE,
+				    CUR_SYMBOL_TYPE, objfile); /* Always */
 	  }
 #endif /* DBXREAD_ONLY */
 	  continue;
@@ -182,15 +183,15 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 		 The first one is a directory name; the second the file name.
 		 If pst exists, is empty, and has a filename ending in '/',
 		 we assume the previous N_SO was a directory name. */
-	      if (global_psymbols.next
-		  ==  (global_psymbols.list + pst->globals_offset)
-		&& static_psymbols.next
-		  == (static_psymbols.list + pst->statics_offset)
+	      if (pst -> objfile -> global_psymbols.next
+		  ==  (pst -> objfile -> global_psymbols.list + pst->globals_offset)
+		&& pst -> objfile -> static_psymbols.next
+		  == (pst -> objfile -> static_psymbols.list + pst->statics_offset)
 		&& pst->filename && pst->filename[0]
 		&& pst->filename[strlen(pst->filename)-1] == '/') {
 		  /* Just replace the directory name with the real filename. */
 		  pst->filename =
-		      (char *) obstack_alloc (psymbol_obstack,
+		      (char *) obstack_alloc (&pst->objfile->psymbol_obstack,
 					      strlen (namestring) + 1);
 		  strcpy (pst->filename, namestring);
 		  continue;
@@ -208,7 +209,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	  pst = START_PSYMTAB (objfile, addr,
 			       namestring, valu,
 			       first_symnum * symbol_size,
-			       global_psymbols.next, static_psymbols.next);
+			       objfile -> global_psymbols.next,
+			       objfile -> static_psymbols.next);
 	  continue;
 	}
 
@@ -232,8 +234,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	      psymtab_include_list = (char **)
 		alloca ((includes_allocated *= 2) *
 			sizeof (char *));
-	      bcopy (orig, psymtab_include_list,
-		     includes_used * sizeof (char *));
+	      memcpy ((PTR)psymtab_include_list, (PTR)orig,
+		      includes_used * sizeof (char *));
 	    }
 
 #endif /* DBXREAD_ONLY */
@@ -276,8 +278,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	      psymtab_include_list = (char **)
 		alloca ((includes_allocated *= 2) *
 			sizeof (char *));
-	      bcopy (orig, psymtab_include_list,
-		     includes_used * sizeof (char *));
+	      memcpy ((PTR)psymtab_include_list, (PTR)orig,
+		      includes_used * sizeof (char *));
 	    }
 	  continue;
 
@@ -299,20 +301,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	    case 'T':
 	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				   STRUCT_NAMESPACE, LOC_TYPEDEF,
-				   static_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->static_psymbols, CUR_SYMBOL_VALUE);
 	      if (p[2] == 't')
 		{
 		  /* Also a typedef with the same name.  */
 		  ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				       VAR_NAMESPACE, LOC_TYPEDEF,
-				       static_psymbols, CUR_SYMBOL_VALUE);
+				       objfile->static_psymbols, CUR_SYMBOL_VALUE);
 		  p += 1;
 		}
 	      goto check_enum;
 	    case 't':
 	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_TYPEDEF,
-				   static_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->static_psymbols, CUR_SYMBOL_VALUE);
 	    check_enum:
 	      /* If this is an enumerated type, we need to
 		 add all the enum constants to the partial symbol
@@ -361,7 +363,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 			 enum constants in psymtabs, just in symtabs.  */
 		      ADD_PSYMBOL_TO_LIST (p, q - p,
 					   VAR_NAMESPACE, LOC_CONST,
-					   static_psymbols, 0);
+					   objfile->static_psymbols, 0);
 		      /* Point past the name.  */
 		      p = q;
 		      /* Skip over the value.  */
@@ -377,7 +379,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	      /* Constant, e.g. from "const" in Pascal.  */
 	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_CONST,
-				   static_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->static_psymbols, CUR_SYMBOL_VALUE);
 	      continue;
 	    default:
 	      /* Skip if the thing following the : is
@@ -418,13 +420,13 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	    case 'c':
 	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_CONST,
-				   static_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->static_psymbols, CUR_SYMBOL_VALUE);
 	      continue;
 	    case 'S':
 	      CUR_SYMBOL_VALUE += addr;		/* Relocate */
 	      ADD_PSYMBOL_ADDR_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_STATIC,
-				   static_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->static_psymbols, CUR_SYMBOL_VALUE);
 	      continue;
 	    case 'G':
 	      CUR_SYMBOL_VALUE += addr;		/* Relocate */
@@ -432,19 +434,19 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 		 wrong.  See the code that reads 'G's for symtabs. */
 	      ADD_PSYMBOL_ADDR_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_STATIC,
-				   global_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->global_psymbols, CUR_SYMBOL_VALUE);
 	      continue;
 
 	    case 't':
 	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_TYPEDEF,
-				   static_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->static_psymbols, CUR_SYMBOL_VALUE);
 	      continue;
 
 	    case 'f':
 	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_BLOCK,
-				   static_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->static_psymbols, CUR_SYMBOL_VALUE);
 	      continue;
 
 	      /* Global functions were ignored here, but now they
@@ -455,7 +457,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	    case 'F':
 	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_BLOCK,
-				   global_psymbols, CUR_SYMBOL_VALUE);
+				   objfile->global_psymbols, CUR_SYMBOL_VALUE);
 	      continue;
 
 	      /* Two things show up here (hopefully); static symbols of
@@ -523,7 +525,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 		      (struct partial_symtab **)
 			alloca ((dependencies_allocated *= 2)
 				* sizeof (struct partial_symtab *));
-		    bcopy (orig, dependency_list,
+		    memcpy ((PTR)dependency_list, (PTR)orig,
 			   (dependencies_used
 			    * sizeof (struct partial_symtab *)));
 #ifdef DEBUG_INFO
