@@ -17,9 +17,9 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-#include <stdio.h>
 #include "defs.h"
 #include "symtab.h"
+#include "expression.h"
 #include "language.h"
 #include "command.h"
 #include "gdbcmd.h"
@@ -35,16 +35,46 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <fcntl.h>
 #include "gdbcore.h"
 #include "regex.h"
+#include "symfile.h"
+#include "objfiles.h"
+
+/* Prototypes for local functions. */
+
+static int
+open_source_file PARAMS ((struct symtab *));
+
+static int
+get_filename_and_charpos PARAMS ((struct symtab *, char **));
+
+static void
+reverse_search_command PARAMS ((char *, int));
+
+static void
+forward_search_command PARAMS ((char *, int));
+
+static void
+line_info PARAMS ((char *, int));
+
+static void
+list_command PARAMS ((char *, int));
+
+static void
+ambiguous_line_spec PARAMS ((struct symtabs_and_lines *));
+
+static void
+source_info PARAMS ((char *, int));
+
+static void
+show_directories PARAMS ((char *, int));
+
+static void
+find_source_lines PARAMS ((struct symtab *, int));
 
 /* If we use this declaration, it breaks because of fucking ANSI "const" stuff
    on some systems.  We just have to not declare it at all, have it default
    to int, and possibly botch on a few systems.  Thanks, ANSIholes... */
 /* extern char *strstr(); */
 
-extern void set_next_address ();
-extern char *basename ();
-extern char *index ();
-
 /* Path of directories to search for source files.
    Same format as the PATH environment variable's value.  */
 
@@ -80,8 +110,8 @@ static int first_line_listed;
    symtab.  Sigh.  Behavior specification: If it is called with a
    non-zero argument, that is the symtab to select.  If it is not,
    first lookup "main"; if it exists, use the symtab and line it
-   defines.  If not, take the last symtab in the symtab_list (if it
-   exists) or the last symtab in the psymtab_list (if *it* exists).  If
+   defines.  If not, take the last symtab in the symtab lists (if it
+   exists) or the last symtab in the psymtab lists (if *it* exists).  If
    none of this works, report an error.   */
 
 void
@@ -92,6 +122,7 @@ select_source_symtab (s)
   struct symtab_and_line sal;
   struct partial_symtab *ps;
   struct partial_symtab *cs_pst = 0;
+  struct objfile *ofp;
   
   if (s)
     {
@@ -117,35 +148,47 @@ select_source_symtab (s)
 
   current_source_line = 1;
 
-  for (s = symtab_list; s; s = s->next)
+  for (ofp = object_files; ofp != NULL; ofp = ofp -> next)
     {
-      char *name = s->filename;
-      int len = strlen (name);
-      if (! (len > 2 && !strcmp (&name[len - 2], ".h")))
-	current_source_symtab = s;
+      for (s = ofp -> symtabs; s; s = s->next)
+	{
+	  char *name = s -> filename;
+	  int len = strlen (name);
+	  if (! (len > 2 && (strcmp (&name[len - 2], ".h") == 0)))
+	    {
+	      current_source_symtab = s;
+	    }
+	}
     }
   if (current_source_symtab)
     return;
 
-  /* Howabout the partial symtab list?  */
+  /* Howabout the partial symbol tables? */
 
-  if (partial_symtab_list)
+  for (ofp = object_files; ofp != NULL; ofp = ofp -> next)
     {
-      ps = partial_symtab_list;
-      while (ps)
+      for (ps = ofp -> psymtabs; ps != NULL; ps = ps -> next)
 	{
-	  char *name = ps->filename;
+	  char *name = ps -> filename;
 	  int len = strlen (name);
-	  if (! (len > 2 && !strcmp (&name[len - 2], ".h")))
-	    cs_pst = ps;
-	  ps = ps->next;
+	  if (! (len > 2 && (strcmp (&name[len - 2], ".h") == 0)))
+	    {
+	      cs_pst = ps;
+	    }
 	}
-      if (cs_pst)
-	if (cs_pst->readin)
-	  fatal ("Internal: select_source_symtab: readin pst found and no symtabs.");
-	else
-	  current_source_symtab = PSYMTAB_TO_SYMTAB (cs_pst);
     }
+  if (cs_pst)
+    {
+      if (cs_pst -> readin)
+	{
+	  fatal ("Internal: select_source_symtab: readin pst found and no symtabs.");
+	}
+      else
+	{
+	  current_source_symtab = PSYMTAB_TO_SYMTAB (cs_pst);
+	}
+    }
+
   if (current_source_symtab)
     return;
 
@@ -153,7 +196,9 @@ select_source_symtab (s)
 }
 
 static void
-show_directories ()
+show_directories (ignore, from_tty)
+     char *ignore;
+     int from_tty;
 {
   printf ("Source directories searched: %s\n", source_path);
 }
@@ -167,18 +212,22 @@ void
 forget_cached_source_info ()
 {
   register struct symtab *s;
+  register struct objfile *objfile;
 
-  for (s = symtab_list; s; s = s->next)
+  for (objfile = object_files; objfile != NULL; objfile = objfile -> next)
     {
-      if (s->line_charpos != 0)
+      for (s = objfile -> symtabs; s != NULL; s = s -> next)
 	{
-	  free (s->line_charpos);
-	  s->line_charpos = 0;
-	}
-      if (s->fullname != 0)
-	{
-	  free (s->fullname);
-	  s->fullname = 0;
+	  if (s -> line_charpos != NULL)
+	    {
+	      mfree (objfile -> md, s -> line_charpos);
+	      s -> line_charpos = NULL;
+	    }
+	  if (s -> fullname != NULL)
+	    {
+	      mfree (objfile -> md, s -> fullname);
+	      s -> fullname = NULL;
+	    }
 	}
     }
 }
@@ -210,7 +259,7 @@ directory_command (dirname, from_tty)
   else
     mod_path (dirname, &source_path);
   if (from_tty)
-    show_directories ();
+    show_directories ((char *)0, from_tty);
   forget_cached_source_info ();
 }
 
@@ -237,9 +286,9 @@ mod_path (dirname, which_path)
       struct stat st;
 
       {
-	char *colon = index (name, ':');
-	char *space = index (name, ' ');
-	char *tab = index (name, '\t');
+	char *colon = strchr (name, ':');
+	char *space = strchr (name, ' ');
+	char *tab = strchr (name, '\t');
 	if (colon == 0 && space == 0 && tab ==  0)
 	  p = dirname = name + strlen (name);
 	else
@@ -323,7 +372,7 @@ mod_path (dirname, which_path)
 		  goto skip_dup;	/* Same dir twice in one cmd */
 		strcpy (p, &p[len+1]);	/* Copy from next \0 or  : */
 	      }
-	    p = index (p, ':');
+	    p = strchr (p, ':');
 	    if (p != 0)
 	      ++p;
 	    else
@@ -359,7 +408,9 @@ mod_path (dirname, which_path)
 
 
 static void
-source_info ()
+source_info (ignore, from_tty)
+     char *ignore;
+     int from_tty;
 {
   register struct symtab *s = current_source_symtab;
 
@@ -431,7 +482,7 @@ openp (path, try_cwd_first, string, mode, prot, filename_opened)
   fd = -1;
   for (p = path; p; p = p1 ? p1 + 1 : 0)
     {
-      p1 = (char *) index (p, ':');
+      p1 = (char *) strchr (p, ':');
       if (p1)
 	len = p1 - p;
       else
@@ -486,13 +537,15 @@ openp (path, try_cwd_first, string, mode, prot, filename_opened)
 
 /* Open a source file given a symtab S.  Returns a file descriptor
    or negative number for error.  */
-int
+
+static int
 open_source_file (s)
      struct symtab *s;
 {
   char *path = source_path;
   char *p;
   int result;
+  char *fullname;
 
   /* Quick way out if we already know its full name */
   if (s->fullname) 
@@ -501,7 +554,7 @@ open_source_file (s)
       if (result >= 0)
         return result;
       /* Didn't work -- free old one, try again. */
-      free (s->fullname);
+      mfree (s->objfile->md, s->fullname);
       s->fullname = NULL;
     }
 
@@ -533,6 +586,12 @@ open_source_file (s)
       if (p != s->filename)
 	result = openp(path, 0, p, O_RDONLY,0, &s->fullname);
     }
+  if (result >= 0)
+    {
+      fullname = s -> fullname;
+      s -> fullname = mstrsave (s -> objfile -> md, s -> fullname);
+      free (fullname);
+    }
   return result;
 }
 
@@ -551,9 +610,11 @@ find_source_lines (s, desc)
   register char *data, *p, *end;
   int nlines = 0;
   int lines_allocated = 1000;
-  int *line_charpos = (int *) xmalloc (lines_allocated * sizeof (int));
+  int *line_charpos;
   long exec_mtime;
 
+  line_charpos = (int *) xmmalloc (s -> objfile -> md,
+				   lines_allocated * sizeof (int));
   if (fstat (desc, &st) < 0)
     perror_with_name (s->filename);
 
@@ -584,19 +645,24 @@ find_source_lines (s, desc)
 	  if (nlines == lines_allocated)
 	    {
 	      lines_allocated *= 2;
-	      line_charpos = (int *) xrealloc (line_charpos,
-					       sizeof (int) * lines_allocated);
+	      line_charpos =
+		  (int *) xmrealloc (s -> objfile -> md, (char *) line_charpos,
+				     sizeof (int) * lines_allocated);
 	    }
 	  line_charpos[nlines++] = p - data;
 	}
     }
   s->nlines = nlines;
-  s->line_charpos = (int *) xrealloc (line_charpos, nlines * sizeof (int));
+  s->line_charpos =
+      (int *) xmrealloc (s -> objfile -> md, (char *) line_charpos,
+			 nlines * sizeof (int));
 }
 
 /* Return the character position of a line LINE in symtab S.
    Return 0 if anything is invalid.  */
 
+#if 0	/* Currently unused */
+
 int
 source_line_charpos (s, line)
      struct symtab *s;
@@ -631,13 +697,16 @@ source_charpos_line (s, chr)
     line = s->nlines;
   return line;
 }
+
+#endif	/* 0 */
+
 
 /* Get full pathname and line number positions for a symtab.
    Return nonzero if line numbers may have changed.
    Set *FULLNAME to actual name of the file as found by `openp',
    or to 0 if the file is not found.  */
 
-int
+static int
 get_filename_and_charpos (s, fullname)
      struct symtab *s;
      char **fullname;
@@ -794,7 +863,7 @@ list_command (arg, from_tty)
   int linenum_beg = 0;
   char *p;
 
-  if (symtab_list == 0 && partial_symtab_list == 0)
+  if (!have_full_symbols () && !have_partial_symbols())
     error ("No symbol table is loaded.  Use the \"file\" command.");
 
   /* Pull in a current source symtab if necessary */
