@@ -1,5 +1,5 @@
 /* Do various things to symbol tables (other than lookup), for GDB.
-   Copyright 1986, 1987, 1989, 1991 Free Software Foundation, Inc.
+   Copyright 1986, 1987, 1989, 1991, 1992 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -18,31 +18,75 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
-#include <stdio.h>
 #include "defs.h"
 #include "symtab.h"
+#include "gdbtypes.h"
 #include "bfd.h"
 #include "symfile.h"
+#include "objfiles.h"
 #include "breakpoint.h"
 #include "command.h"
 #include "obstack.h"
 
 #include <string.h>
+
+#ifndef DEV_TTY
+#define DEV_TTY "/dev/tty"
+#endif
+
+/* Prototypes for local functions */
+
+static void 
+dump_symtab PARAMS ((struct objfile *, struct symtab *, FILE *));
+
+static void 
+dump_psymtab PARAMS ((struct objfile *, struct partial_symtab *, FILE *));
+
+static void 
+dump_msymbols PARAMS ((struct objfile *, FILE *));
+
+static void 
+dump_objfile PARAMS ((struct objfile *));
+
+static void
+printobjfiles_command PARAMS ((char *, int));
+
+static int
+block_depth PARAMS ((struct block *));
+
+static void
+print_partial_symbol PARAMS ((struct partial_symbol *, int, char *, FILE *));
+
+static void
+printpsyms_command PARAMS ((char *, int));
+
+static void
+print_symbol PARAMS ((struct symbol *, int, FILE *));
+
+static void
+printsyms_command PARAMS ((char *, int));
+
+static void
+free_symtab_block PARAMS ((struct objfile *, struct block *));
+
+static void
+printmsyms_command PARAMS ((char *, int));
 
 /* Free a struct block <- B and all the symbols defined in that block.  */
 
 static void
-free_symtab_block (b)
+free_symtab_block (objfile, b)
+     struct objfile *objfile;
      struct block *b;
 {
   register int i, n;
   n = BLOCK_NSYMS (b);
   for (i = 0; i < n; i++)
     {
-      free (SYMBOL_NAME (BLOCK_SYM (b, i)));
-      free (BLOCK_SYM (b, i));
+      mfree (objfile -> md, SYMBOL_NAME (BLOCK_SYM (b, i)));
+      mfree (objfile -> md, (PTR) BLOCK_SYM (b, i));
     }
-  free (b);
+  mfree (objfile -> md, (PTR) b);
 }
 
 /* Free all the storage associated with the struct symtab <- S.
@@ -75,9 +119,9 @@ free_symtab (s)
       bv = BLOCKVECTOR (s);
       n = BLOCKVECTOR_NBLOCKS (bv);
       for (i = 0; i < n; i++)
-	free_symtab_block (BLOCKVECTOR_BLOCK (bv, i));
+	free_symtab_block (s -> objfile, BLOCKVECTOR_BLOCK (bv, i));
       /* Free the blockvector itself.  */
-      free (bv);
+      mfree (s -> objfile -> md, (PTR) bv);
       /* Also free the linetable.  */
       
     case free_linetable:
@@ -85,49 +129,240 @@ free_symtab (s)
 	 or by some other symtab, except for our linetable.
 	 Free that now.  */
       if (LINETABLE (s))
-	free (LINETABLE (s));
+	mfree (s -> objfile -> md, (PTR) LINETABLE (s));
       break;
     }
 
   /* If there is a single block of memory to free, free it.  */
-  if (s->free_ptr)
-    free (s->free_ptr);
+  if (s -> free_ptr != NULL)
+    mfree (s -> objfile -> md, s -> free_ptr);
 
   /* Free source-related stuff */
-  if (s->line_charpos)
-    free (s->line_charpos);
-  if (s->fullname)
-    free (s->fullname);
-  free (s);
+  if (s -> line_charpos != NULL)
+    mfree (s -> objfile -> md, (PTR) s -> line_charpos);
+  if (s -> fullname != NULL)
+    mfree (s -> objfile -> md, s -> fullname);
+  mfree (s -> objfile -> md, (PTR) s);
 }
-
-static int block_depth ();
-static void print_symbol ();
-static void print_partial_symbol ();
 
-void
-printsyms_command (filename)
-     char *filename;
+static void 
+dump_objfile (objfile)
+     struct objfile *objfile;
+{
+  struct symtab *symtab;
+  struct partial_symtab *psymtab;
+
+  printf_filtered ("\nObject file %s:  ", objfile -> name);
+  printf_filtered ("Objfile at %x, bfd at %x, %d minsyms\n\n",
+		   objfile, objfile -> obfd, objfile->minimal_symbol_count);
+
+  if (objfile -> psymtabs)
+    {
+      printf_filtered ("Psymtabs:\n");
+      for (psymtab = objfile -> psymtabs;
+	   psymtab != NULL;
+	   psymtab = psymtab -> next)
+	{
+	  printf_filtered ("%s at %x, ", psymtab -> filename, psymtab);
+	  if (psymtab -> objfile != objfile)
+	    {
+	      printf_filtered ("NOT ON CHAIN!  ");
+	    }
+	  wrap_here ("  ");
+	}
+      printf_filtered ("\n\n");
+    }
+
+  if (objfile -> symtabs)
+    {
+      printf_filtered ("Symtabs:\n");
+      for (symtab = objfile -> symtabs;
+	   symtab != NULL;
+	   symtab = symtab->next)
+	{
+	  printf_filtered ("%s at %x, ", symtab -> filename, symtab);
+	  if (symtab -> objfile != objfile)
+	    {
+	      printf_filtered ("NOT ON CHAIN!  ");
+	    }
+	  wrap_here ("  ");
+	}
+      printf_filtered ("\n\n");
+    }
+}
+
+/* Print minimal symbols from this objfile.  */
+ 
+static void 
+dump_msymbols (objfile, outfile)
+     struct objfile *objfile;
+     FILE *outfile;
+{
+  struct minimal_symbol *msymbol;
+  int index;
+  char ms_type;
+  
+  fprintf_filtered (outfile, "\nObject file %s:\n\n", objfile -> name);
+  for (index = 0, msymbol = objfile -> msymbols;
+       msymbol -> name != NULL; msymbol++, index++)
+    {
+      switch (msymbol -> type)
+	{
+	  case mst_unknown:
+	    ms_type = 'u';
+	    break;
+	  case mst_text:
+	    ms_type = 't';
+	    break;
+	  case mst_data:
+	    ms_type = 'd';
+	    break;
+	  case mst_bss:
+	    ms_type = 'b';
+	    break;
+	  case mst_abs:
+	    ms_type = 'a';
+	    break;
+	  default:
+	    ms_type = '?';
+	    break;
+	}
+      fprintf_filtered (outfile, "[%2d] %c %#10x %s\n", index, ms_type,
+			msymbol -> address, msymbol -> name);
+    }
+  if (objfile -> minimal_symbol_count != index)
+    {
+      warning ("internal error:  minimal symbol count %d != %d",
+	       objfile -> minimal_symbol_count, index);
+    }
+  fprintf_filtered (outfile, "\n");
+}
+
+static void
+dump_psymtab (objfile, psymtab, outfile)
+     struct objfile *objfile;
+     struct partial_symtab *psymtab;
+     FILE *outfile;
+{
+
+  fprintf_filtered (outfile, "\nPartial symtab for source file %s ",
+		    psymtab -> filename);
+  fprintf_filtered (outfile, "(object 0x%x)\n\n", psymtab);
+  fprintf (outfile, "  Read from object file %s (0x%x)\n",
+	   objfile -> name, objfile);
+  
+  if (psymtab -> readin)
+    {
+      fprintf_filtered (outfile,
+		"  Full symtab was read (at 0x%x by function at 0x%x)\n",
+			psymtab -> symtab, psymtab -> read_symtab);
+    }
+  fprintf_filtered (outfile, "  Relocate symbols by 0x%x\n",
+		    psymtab -> addr);
+  fprintf_filtered (outfile, "  Symbols cover text addresses 0x%x-0x%x\n",
+		    psymtab -> textlow, psymtab -> texthigh);
+  fprintf_filtered (outfile, "  Depends on %d other partial symtabs.\n",
+		    psymtab -> number_of_dependencies);
+  if (psymtab -> n_global_syms > 0)
+    {
+      print_partial_symbol (objfile -> global_psymbols.list
+			    + psymtab -> globals_offset,
+			    psymtab -> n_global_syms, "Global", outfile);
+    }
+  if (psymtab -> n_static_syms > 0)
+    {
+      print_partial_symbol (objfile -> static_psymbols.list
+			    + psymtab -> statics_offset,
+			    psymtab -> n_static_syms, "Static", outfile);
+    }
+  fprintf_filtered (outfile, "\n");
+}
+
+static void 
+dump_symtab (objfile, symtab, outfile)
+     struct objfile *objfile;
+     struct symtab *symtab;
+     FILE *outfile;
 {
-  FILE *outfile;
-  register struct symtab *s;
   register int i, j;
   int len, blen;
   register struct linetable *l;
   struct blockvector *bv;
   register struct block *b;
   int depth;
+
+  fprintf (outfile, "\nSymtab for file %s\n", symtab->filename);
+  fprintf (outfile, "Read from object file %s (%x)\n", objfile->name,
+	   objfile);
+  
+  /* First print the line table.  */
+  l = LINETABLE (symtab);
+  if (l) {
+    fprintf (outfile, "\nLine table:\n\n");
+    len = l->nitems;
+    for (i = 0; i < len; i++)
+      fprintf (outfile, " line %d at %x\n", l->item[i].line,
+	       l->item[i].pc);
+  }
+  /* Now print the block info.  */
+  fprintf (outfile, "\nBlockvector:\n\n");
+  bv = BLOCKVECTOR (symtab);
+  len = BLOCKVECTOR_NBLOCKS (bv);
+  for (i = 0; i < len; i++)
+    {
+      b = BLOCKVECTOR_BLOCK (bv, i);
+      depth = block_depth (b) * 2;
+      print_spaces (depth, outfile);
+      fprintf (outfile, "block #%03d (object 0x%x) ", i, b);
+      fprintf (outfile, "[0x%x..0x%x]", BLOCK_START (b), BLOCK_END (b));
+      if (BLOCK_SUPERBLOCK (b))
+	fprintf (outfile, " (under 0x%x)", BLOCK_SUPERBLOCK (b));
+      if (BLOCK_FUNCTION (b))
+	fprintf (outfile, " %s", SYMBOL_NAME (BLOCK_FUNCTION (b)));
+      fputc ('\n', outfile);
+      blen = BLOCK_NSYMS (b);
+      for (j = 0; j < blen; j++)
+	{
+	  print_symbol (BLOCK_SYM (b, j), depth + 1, outfile);
+	}
+    }
+  fprintf (outfile, "\n");
+}
+
+static void
+printsyms_command (args, from_tty)
+     char *args;
+     int from_tty;
+{
+  char **argv;
+  FILE *outfile;
   struct cleanup *cleanups;
-  extern int fclose();
-  char *symname;
+  char *symname = NULL;
+  char *filename = DEV_TTY;
+  struct objfile *objfile;
+  struct symtab *s;
 
-  if (filename == 0)
-    error_no_arg ("file to write symbol data in");
+  dont_repeat ();
 
-  /* If a second arg is supplied, it is a source file name to match on */
-  symname = strchr (filename, ' ');
-  if (symname)
-    *symname++ = '\0';
+  if (args == NULL)
+    {
+      error ("printsyms takes an output file name and optional symbol file name");
+    }
+  else if ((argv = buildargv (args)) == NULL)
+    {
+      nomem (0);
+    }
+  cleanups = make_cleanup (freeargv, (char *) argv);
+
+  if (argv[0] != NULL)
+    {
+      filename = argv[0];
+      /* If a second arg is supplied, it is a source file name to match on */
+      if (argv[1] != NULL)
+	{
+	  symname = argv[1];
+	}
+    }
 
   filename = tilde_expand (filename);
   make_cleanup (free, filename);
@@ -135,56 +370,12 @@ printsyms_command (filename)
   outfile = fopen (filename, "w");
   if (outfile == 0)
     perror_with_name (filename);
+  make_cleanup (fclose, (char *) outfile);
 
-  cleanups = make_cleanup (fclose, outfile);
   immediate_quit++;
-
-  for (s = symtab_list; s; s = s->next)
-    {
-      /* If source file name is specified, reject all but that one.  */
-      if (symname)
-        if (0 != strncmp (symname, s->filename, strlen (symname)))
-	  continue;
-
-      fprintf (outfile, "Symtab for file %s\n", s->filename);
-      fprintf (outfile, "Read from object file %s (%x)\n", s->objfile->name,
-			s->objfile);
-
-      /* First print the line table.  */
-      l = LINETABLE (s);
-      if (l) {
-	fprintf (outfile, "\nLine table:\n\n");
-	len = l->nitems;
-	for (i = 0; i < len; i++)
-	  fprintf (outfile, " line %d at %x\n", l->item[i].line,
-		   l->item[i].pc);
-      }
-      /* Now print the block info.  */
-      fprintf (outfile, "\nBlockvector:\n\n");
-      bv = BLOCKVECTOR (s);
-      len = BLOCKVECTOR_NBLOCKS (bv);
-      for (i = 0; i < len; i++)
-	{
-	  b = BLOCKVECTOR_BLOCK (bv, i);
-	  depth = block_depth (b) * 2;
-	  print_spaces (depth, outfile);
-	  fprintf (outfile, "block #%03d (object 0x%x) ", i, b);
-	  fprintf (outfile, "[0x%x..0x%x]", BLOCK_START (b), BLOCK_END (b));
-	  if (BLOCK_SUPERBLOCK (b))
-	    fprintf (outfile, " (under 0x%x)", BLOCK_SUPERBLOCK (b));
-	  if (BLOCK_FUNCTION (b))
-	    fprintf (outfile, " %s", SYMBOL_NAME (BLOCK_FUNCTION (b)));
-	  fputc ('\n', outfile);
-	  blen = BLOCK_NSYMS (b);
-	  for (j = 0; j < blen; j++)
-	    {
-	      print_symbol (BLOCK_SYM (b, j), depth + 1, outfile);
-	    }
-	}
-
-      fprintf (outfile, "\n\n");
-    }
-
+  ALL_SYMTABS (objfile, s)
+    if (symname == NULL || (strcmp (symname, s -> filename) == 0))
+      dump_symtab (objfile, s, outfile);
   immediate_quit--;
   do_cleanups (cleanups);
 }
@@ -301,23 +492,40 @@ print_symbol (symbol, depth, outfile)
   fprintf (outfile, "\n");
 }
 
-void
-printpsyms_command (filename)
-     char *filename;
+static void
+printpsyms_command (args, from_tty)
+     char *args;
+     int from_tty;
 {
+  char **argv;
   FILE *outfile;
-  struct partial_symtab *p;
   struct cleanup *cleanups;
-  extern int fclose();
-  char *symname;
+  char *symname = NULL;
+  char *filename = DEV_TTY;
+  struct objfile *objfile;
+  struct partial_symtab *ps;
 
-  if (filename == 0)
-    error_no_arg ("file to write partial symbol data in");
+  dont_repeat ();
 
-  /* If a second arg is supplied, it is a source file name to match on */
-  symname = strchr (filename, ' ');
-  if (symname)
-    *symname++ = '\0';
+  if (args == NULL)
+    {
+      error ("printpsyms takes an output file name and optional symbol file name");
+    }
+  else if ((argv = buildargv (args)) == NULL)
+    {
+      nomem (0);
+    }
+  cleanups = make_cleanup (freeargv, (char *) argv);
+
+  if (argv[0] != NULL)
+    {
+      filename = argv[0];
+      /* If a second arg is supplied, it is a source file name to match on */
+      if (argv[1] != NULL)
+	{
+	  symname = argv[1];
+	}
+    }
 
   filename = tilde_expand (filename);
   make_cleanup (free, filename);
@@ -325,50 +533,22 @@ printpsyms_command (filename)
   outfile = fopen (filename, "w");
   if (outfile == 0)
     perror_with_name (filename);
+  make_cleanup (fclose, outfile);
 
-  cleanups = make_cleanup (fclose, outfile);
   immediate_quit++;
-
-  for (p = partial_symtab_list; p; p = p->next)
-    {
-      /* If source file name is specified, reject all but that one.  */
-      if (symname)
-        if (0 != strncmp (symname, p->filename, strlen (symname)))
-	  continue;
-
-      fprintf_filtered (outfile, "Partial symtab for source file %s ",
-	       p->filename);
-      fprintf_filtered (outfile, "(object 0x%x)\n\n", p);
-      fprintf (outfile, "  Read from object file %s (0x%x)\n", p->objfile->name,
-			p->objfile);
-
-      if (p->readin)
-	fprintf_filtered (outfile, "  Full symtab was read (at 0x%x by function at 0x%x)\n",
-			  p->symtab, p->read_symtab);
-      fprintf_filtered (outfile, "  Relocate symbols by 0x%x\n", p->addr);
-      fprintf_filtered (outfile, "  Symbols cover text addresses 0x%x-0x%x\n",
-			p->textlow, p->texthigh);
-      fprintf_filtered (outfile, "  Depends on %d other partial symtabs.\n",
-			p->number_of_dependencies);
-      if (p->n_global_syms > 0)
-	print_partial_symbol (global_psymbols.list + p->globals_offset,
-			      p->n_global_syms, "Global", outfile);
-      if (p->n_static_syms > 0)
-	print_partial_symbol (static_psymbols.list + p->statics_offset,
-			      p->n_static_syms, "Static", outfile);
-      fprintf_filtered (outfile, "\n\n");
-    }
-
+  ALL_PSYMTABS (objfile, ps)
+    if (symname == NULL || (strcmp (symname, ps -> filename) == 0))
+      dump_psymtab (objfile, ps, outfile);
   immediate_quit--;
   do_cleanups (cleanups);
 }
 
 static void
 print_partial_symbol (p, count, what, outfile)
-struct partial_symbol *p;
-int count;
-char *what;
-FILE *outfile;
+     struct partial_symbol *p;
+     int count;
+     char *what;
+     FILE *outfile;
 {
 
   fprintf_filtered (outfile, "  %s partial symbols:\n", what);
@@ -444,6 +624,72 @@ FILE *outfile;
     }
 }
 
+static void
+printmsyms_command (args, from_tty)
+     char *args;
+     int from_tty;
+{
+  char **argv;
+  FILE *outfile;
+  struct cleanup *cleanups;
+  char *filename = DEV_TTY;
+  char *symname = NULL;
+  struct objfile *objfile;
+
+  dont_repeat ();
+
+  if (args == NULL)
+    {
+      error ("printmsyms takes an output file name and optional symbol file name");
+    }
+  else if ((argv = buildargv (args)) == NULL)
+    {
+      nomem (0);
+    }
+  cleanups = make_cleanup (freeargv, argv);
+
+  if (argv[0] != NULL)
+    {
+      filename = argv[0];
+      /* If a second arg is supplied, it is a source file name to match on */
+      if (argv[1] != NULL)
+	{
+	  symname = argv[1];
+	}
+    }
+
+  filename = tilde_expand (filename);
+  make_cleanup (free, filename);
+  
+  outfile = fopen (filename, "w");
+  if (outfile == 0)
+    perror_with_name (filename);
+  make_cleanup (fclose, outfile);
+
+  immediate_quit++;
+  ALL_OBJFILES (objfile)
+    if (symname == NULL || (strcmp (symname, objfile -> name) == 0))
+      dump_msymbols (objfile, outfile);
+  immediate_quit--;
+  fprintf_filtered (outfile, "\n\n");
+  do_cleanups (cleanups);
+}
+
+static void
+printobjfiles_command (ignore, from_tty)
+     char *ignore;
+     int from_tty;
+{
+  struct objfile *objfile;
+
+  dont_repeat ();
+
+  immediate_quit++;
+  ALL_OBJFILES (objfile)
+    dump_objfile (objfile);
+  immediate_quit--;
+}
+
 /* Return the nexting depth of a block within other blocks in its symtab.  */
 
 static int
@@ -454,111 +700,28 @@ block_depth (block)
   while (block = BLOCK_SUPERBLOCK (block)) i++;
   return i;
 }
+
 
-static void
-printobjfiles_command ()
-{
-  struct objfile *objfile;
-  struct symtab *symtab;
-  struct partial_symtab *psymtab;
-  int first;
-
-  for (objfile = object_files;  objfile;  objfile = objfile->next) {
-    printf_filtered ("\nObject file %s:  ", objfile->name);
-    printf_filtered ("Objfile at %x, bfd at %x\n\n", objfile, objfile->obfd);
-
-    if (objfile->psymtabs) {
-      printf_filtered ("Psymtabs:\n");
-      for (psymtab = objfile->psymtabs;
-	   psymtab;
-	   psymtab = psymtab->objfile_chain) {
-        printf_filtered ("%s at %x, ", psymtab->filename, psymtab);
-	if (psymtab->objfile != objfile)
-	  printf_filtered ("NOT ON CHAIN!  ");
-        wrap_here ("  ");
-      }
-      printf_filtered ("\n\n");
-    }
-
-    if (objfile->symtabs) {
-      printf_filtered ("Symtabs:\n");
-      for (symtab = objfile->symtabs;
-	   symtab;
-	   symtab = symtab->objfile_chain) {
-        printf_filtered ("%s at %x, ", symtab->filename, symtab);
-	if (symtab->objfile != objfile)
-	  printf_filtered ("NOT ON CHAIN!  ");
-        wrap_here ("  ");
-      }
-      printf_filtered ("\n\n");
-    }
-  }
-
-  /* Now check for psymtabs that aren't owned by an objfile.  */
-
-  first = 1;
-  for (psymtab = partial_symtab_list; psymtab; psymtab = psymtab->next) {
-    for (objfile = object_files; objfile; objfile = objfile->next) {
-      if (psymtab->objfile == objfile)
-        goto next;
-    }
-    if (first)
-      printf_filtered ("Psymtabs that aren't owned by any objfile:\n");
-    first = 0;
-    printf_filtered ("  %s at %x, psymtab->objfile %x\n", psymtab->filename,
-		     psymtab, psymtab->objfile);
-  next: ;
-  }
-
-  /* Now check for symtabs that aren't owned by an objfile.  */
-
-  first = 1;
-  for (symtab = symtab_list; symtab; symtab = symtab->next) {
-    for (objfile = object_files; objfile; objfile = objfile->next) {
-      if (symtab->objfile == objfile)
-        goto next2;
-    }
-    if (first)
-      printf_filtered ("Symtabs that aren't owned by any objfile:\n");
-    first = 0;
-    printf_filtered ("  %s at %x, symtab->objfile %x\n", symtab->filename,
-		     symtab, symtab->objfile);
-  next2: ;
-  }
-}
-
-const struct cplus_struct_type cplus_struct_default;
-
-void
-allocate_cplus_struct_type (type)
-     struct type *type;
-{
-  if (!HAVE_CPLUS_STRUCT (type))
-    {
-      TYPE_CPLUS_SPECIFIC (type) = (struct cplus_struct_type *)
-	obstack_alloc (symbol_obstack, sizeof (struct cplus_struct_type));
-      *(TYPE_CPLUS_SPECIFIC(type)) = cplus_struct_default;
-    }
-}
-
 /* Increase the space allocated for LISTP. */
 
 void
-extend_psymbol_list(listp)
+extend_psymbol_list (listp, objfile)
      register struct psymbol_allocation_list *listp;
+     struct objfile *objfile;
 {
   int new_size;
   if (listp->size == 0)
     {
       new_size = 255;
       listp->list = (struct partial_symbol *)
-	xmalloc (new_size * sizeof (struct partial_symbol));
+	xmmalloc (objfile -> md, new_size * sizeof (struct partial_symbol));
     }
   else
     {
       new_size = listp->size * 2;
       listp->list = (struct partial_symbol *)
-	xrealloc (listp->list, new_size * sizeof (struct partial_symbol));
+	xmrealloc (objfile -> md, (char *) listp->list,
+		   new_size * sizeof (struct partial_symbol));
     }
   /* Next assumes we only went one over.  Should be good if
      program works correctly */
@@ -584,9 +747,9 @@ add_psymbol_to_list (name, namelength, namespace, class, listp, psymval)
   register struct partial_symbol *psym;
 
   if (listp -> next >= listp -> list + listp -> size)
-    extend_psymbol_list (listp);
+    extend_psymbol_list (listp, objfile);
   psym = listp -> next++;
-  SYMBOL_NAME (psym) = (char *) obstack_alloc (psymbol_obstack,
+  SYMBOL_NAME (psym) = (char *) obstack_alloc (&objfile->psymbol_obstack,
 					       namelength + 1);
   memcpy (SYMBOL_NAME (psym), name, namelength);
   SYMBOL_NAME (psym)[namelength] = '\0';
@@ -611,9 +774,9 @@ add_psymbol_addr_to_list (name, namelength, namespace, class, listp, psymval)
   register struct partial_symbol *psym;
 
   if (listp -> next >= listp -> list + listp -> size)
-    extend_psymbol_list (listp);
+    extend_psymbol_list (listp, objfile);
   psym = listp -> next++;
-  SYMBOL_NAME (psym) = (char *) obstack_alloc (psymbol_obstack,
+  SYMBOL_NAME (psym) = (char *) obstack_alloc (&objfile->psymbol_obstack,
 					       namelength + 1);
   memcpy (SYMBOL_NAME (psym), name, namelength);
   SYMBOL_NAME (psym)[namelength] = '\0';
@@ -627,15 +790,15 @@ add_psymbol_addr_to_list (name, namelength, namespace, class, listp, psymval)
 void
 _initialize_symmisc ()
 {
-  symtab_list = (struct symtab *) 0;
-  partial_symtab_list = (struct partial_symtab *) 0;
-  
-  add_com ("printsyms", class_obscure, printsyms_command,
-	   "Print dump of current symbol definitions to file OUTFILE.\n\
+  add_com ("printmsyms", class_obscure, printmsyms_command,
+	   "Print dump of current minimal symbol definitions to file OUTFILE.\n\
 If a SOURCE file is specified, dump only that file's symbols.");
   add_com ("printpsyms", class_obscure, printpsyms_command,
 	  "Print dump of current partial symbol definitions to file OUTFILE.\n\
 If a SOURCE file is specified, dump only that file's partial symbols.");
+  add_com ("printsyms", class_obscure, printsyms_command,
+	   "Print dump of current symbol definitions to file OUTFILE.\n\
+If a SOURCE file is specified, dump only that file's symbols.");
   add_com ("printobjfiles", class_obscure, printobjfiles_command,
 	   "Print dump of current object file definitions.");
 }
