@@ -1,5 +1,5 @@
 /* Perform arithmetic and other operations on values, for GDB.
-   Copyright 1986, 1989, 1991 Free Software Foundation, Inc.
+   Copyright 1986, 1989, 1991, 1992 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -19,14 +19,16 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "defs.h"
 #include "value.h"
+#include "symtab.h"
+#include "gdbtypes.h"
 #include "expression.h"
 #include "target.h"
 #include <string.h>
 
-
-value value_x_binop ();
-value value_subscripted_rvalue ();
+static value
+value_subscripted_rvalue PARAMS ((value, value));
 
+
 value
 value_add (arg1, arg2)
 	value arg1, arg2;
@@ -119,7 +121,7 @@ value_subscript (array, idx)
    (eg, a vector register).  This routine used to promote floats
    to doubles, but no longer does.  */
 
-value
+static value
 value_subscripted_rvalue (array, idx)
      value array, idx;
 {
@@ -270,11 +272,11 @@ value_x_binop (arg1, arg2, op, otherop)
 	  argvec[1] = argvec[0];
 	  argvec++;
 	}
-      return target_call_function (argvec[0], 2 - static_memfuncp, argvec + 1);
+      return call_function_by_hand (argvec[0], 2 - static_memfuncp, argvec + 1);
     }
   error ("member function %s not found", tstr);
 #ifdef lint
-  return target_call_function (argvec[0], 2 - static_memfuncp, argvec + 1);
+  return call_function_by_hand (argvec[0], 2 - static_memfuncp, argvec + 1);
 #endif
 }
 
@@ -329,7 +331,7 @@ value_x_unop (arg1, op)
 	  argvec[1] = argvec[0];
 	  argvec++;
 	}
-      return target_call_function (argvec[0], 1 - static_memfuncp, argvec + 1);
+      return call_function_by_hand (argvec[0], 1 - static_memfuncp, argvec + 1);
     }
   error ("member function %s not found", tstr);
   return 0;  /* For lint -- never reached */
